@@ -5,6 +5,8 @@ extern "C" {
 #endif
 
 void	init_auto_menu( void );
+void	reset_static_auto_menu( void );
+void	reset_ls_flg_ena_auto_mark();
 void	init_auto_mark_all_appoint( void );
 void	init_auto_mark_appoint( long ls_mbr_n );
 void	init_auto_mark_all_mbr( void );
@@ -22,6 +24,7 @@ bool_t	exec_menu_auto_mark( mbr_t *mbr );
 bool_t	exec_menu_auto_mark_mnstr( mbr_t *mbr );
 bool_t	exec_menu_auto_mark_item( mbr_t *mbr );
 bool_t	exec_menu_auto_mark_door( mbr_t *mbr );
+bool_t	exec_menu_auto_mark_trap( mbr_t *mbr );
 bool_t	exec_menu_auto_mark_mbr( mbr_t *mbr );
 char	*cb_menu_auto_mark_edit( menu_t **pp, menu_t **dflt );
 char	*cb_menu_auto_mark_edit_init(
@@ -65,10 +68,16 @@ char	*cb_menu_auto_mark_paste_large( menu_t **pp, menu_t **dflt );
 char	*cb_menu_auto_mark_copy_small( menu_t **pp, menu_t **dflt );
 char	*cb_menu_auto_mark_paste_small( menu_t **pp, menu_t **dflt );
 char	*cb_menu_auto_mark_edit_name( menu_t **pp, menu_t **dflt );
+char	*cb_menu_auto_mark_remove( menu_t **pp, menu_t **dflt );
+char	*cb_menu_auto_mark_arrange_up( menu_t **pp, menu_t **dflt );
+char	*cb_menu_auto_mark_arrange_down( menu_t **pp, menu_t **dflt );
 char	*cb_menu_auto_mark_arrange_top( menu_t **pp, menu_t **dflt );
 char	*cb_menu_auto_mark_arrange_bottom( menu_t **pp, menu_t **dflt );
+char	*menu_auto_mark_arrange_1( long n, long d );
 char	*menu_auto_mark_arrange( long n, long d );
+bool_t	arrange_auto_mark_1( long n, long d );
 bool_t	arrange_auto_mark( long n, long d );
+bool_t	arrange_auto_mark_mbr_1( long n, long d );
 bool_t	arrange_auto_mark_mbr( long ls_mbr_n, long d );
 bool_t	remove_auto_mark_mbr( long ls_mbr_n );
 char	*get_msg_menu_auto_mark_edit_trgt( mbr_t *mbr );
@@ -82,9 +91,12 @@ char	*get_msg_ttl_auto_mark_name( long n );
 char	*get_msg_ttl_auto_mark_info( long n );
 char	*get_msg_ttl_auto_mark_hint( long n );
 auto_mark_t	*get_ls_auto_mark( long ls_mbr_n );
+bool_t	chk_flg_ena_auto_mark( long n );
+void	set_flg_ena_auto_mark( long n, bool_t flg );
 auto_mark_appoint_item_t	*get_ls_auto_mark_appoint_item(
 	long ls_mbr_n, long n
 );
+void	check_memory_amenu_c( void );
 
 #ifdef __cplusplus
 }
