@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: call.cpp,v 1.108 2009/07/25 20:47:15 bowkenken Exp $
+* $Id: call.cpp,v 1.109 2009/08/08 19:01:31 bowkenken Exp $
 ***************************************************************/
 
 #define	CALL_CPP
@@ -1298,7 +1298,11 @@ void	call_demo_last_boss_reset( void )
 	if( gPcgDun.pDemoLastBoss == NULL )
 		return;
 
+	gui_begin();
+
 	gPcgDun.pDemoLastBoss->reset();
+
+	gui_end();
 #endif
 }
 
@@ -1417,17 +1421,13 @@ void	call_game_sound_stop( void )
 long	call_game_joystick_get_joy( void )
 {
 #if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
-	{
-		long	n;
+	gui_begin();
 
-		gui_begin();
+	long n = gJoystick.getJoy();
 
-		n = gJoystick.getJoy();
+	gui_end();
 
-		gui_end();
-
-		return n;
-	}
+	return n;
 #endif
 
 	return g_joy_n;
@@ -1460,17 +1460,13 @@ void	call_game_joystick_sel_joy( long n )
 const char	*call_game_joystick_get_name( long n )
 {
 #if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
-	{
-		const char	*s;
+	gui_begin();
 
-		gui_begin();
+	const char	*s = gJoystick.getName( n );
 
-		s = gJoystick.getName( n );
+	gui_end();
 
-		gui_end();
-
-		return s;
-	}
+	return s;
 #endif
 
 	return NULL;
@@ -1484,17 +1480,13 @@ const char	*call_game_joystick_get_name( long n )
 long	call_game_joystick_get_kind( void )
 {
 #if	defined( D_WS ) || defined( D_GTK ) || defined( D_MFC )
-	{
-		long	n;
+	gui_begin();
 
-		gui_begin();
+	long n = gJoystick.getKind();
 
-		n = gJoystick.getKind();
+	gui_end();
 
-		gui_end();
-
-		return n;
-	}
+	return n;
 #endif
 
 	return g_joy_kind;
