@@ -20,6 +20,7 @@ void	last_boss_phase( void );
 void	draw_phase( bool_t flg_first_phase );
 void	move_phase_chr( chr_t *chr );
 void	act_phase_chr( chr_t *chr );
+void	dismissal_phase_chr( chr_t *chr );
 bool_t	chk_can_act_chr( chr_t *chr );
 bool_t	chk_bash_or_move( chr_t *chr );
 void	bash_or_move( chr_t *chr );
@@ -242,6 +243,7 @@ bool_t	resi_roll(
 	chr_t *d_p, resi_kind_t d_resi
 );
 long	roll_dice( long n, bool_t flg_add );
+void	check_memory_chr_c( void );
 
 #ifdef __cplusplus
 }
