@@ -5,6 +5,7 @@ extern "C" {
 #endif
 
 void	init_curs( void );
+void	reset_static_curs( void );
 bool_t	init_curs_color( void );
 bool_t	init_curs_color_pair( curs_attr_t *p, short pair_n );
 bool_t	chk_curs_draw( void );
@@ -38,12 +39,25 @@ void	curs_clear( void );
 int	curs_getch( void );
 void	curs_getnstr( char *s, int len );
 bool_t	chk_color( void );
-long	get_max_line( void );
+long	get_screen_col( void );
+long	get_screen_row( void );
+long	get_message_col( void );
+long	get_message_row( void );
+long	get_map_col( void );
+long	get_map_row( void );
 bool_t	chk_flg_screen_size_auto( void );
 void	set_flg_screen_size_auto( bool_t flg );
-long	get_screen_size_lines( void );
-void	set_screen_size_lines( long n );
+long	get_screen_size_col( void );
+void	set_screen_size_col( long n );
+long	get_screen_size_row( void );
+void	set_screen_size_row( long n );
+long	get_screen_size_message( void );
+void	set_screen_size_message( long n );
+bool_t	chk_flg_resize_map( void );
+void	set_flg_resize_map( bool_t flg );
+bool_t	cmp_attr( curs_attr_t *a, curs_attr_t *b );
 int	attr_dos( curs_attr_t *attr );
+void	check_memory_curs_c( void );
 
 #ifdef __cplusplus
 }
