@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2009 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: draw.h,v 1.59 2008/05/05 12:43:44 bowkenken Exp $
+* $Id: draw.h,v 1.71 2009/08/16 04:20:27 bowkenken Exp $
 ***************************************************************/
 
 #ifndef	DRAW_H
@@ -114,12 +114,12 @@
 #define	WIN_REQUEST_DRAW_WIDTH	40
 #define	WIN_REQUEST_DRAW_MIN_Y	4
 
-#define	WIN_STAT_DRAW_MIN_X	(WIN_MAP_DRAW_MAX_X + 1 + 1)
+#define	WIN_STAT_DRAW_MIN_X	(get_map_col() + 1)
 #define	WIN_STAT_DRAW_MIN_Y	0
-#define	WIN_STAT_DRAW_WIDTH	\
-		(WIN_STAT_DRAW_MAX_X - WIN_STAT_DRAW_MIN_X + 1)
+#define	WIN_STAT_DRAW_WIDTH	(SCREEN_WIDTH / 2 - 2)
 #define	WIN_STAT_DRAW_HEIGHT	3
-#define	WIN_STAT_DRAW_MAX_X	(SCREEN_WIDTH - 1)
+#define	WIN_STAT_DRAW_MAX_X	\
+		(WIN_STAT_DRAW_MIN_X + WIN_STAT_DRAW_WIDTH - 1)
 #define	WIN_STAT_DRAW_MAX_Y	(WIN_STAT_DRAW_MIN_Y \
 		+ ((MBR_MAX_N) * WIN_STAT_DRAW_HEIGHT - 1) - 1)
 
@@ -166,8 +166,12 @@ typedef unsigned long	flg_msg_t;
 /* åκԿ */
 #define	LS_MSG_MAX_N	(LS_MSG_DRAW_N * 8)
 
+/* åκʸ */
+#define	LS_MSG_MAX_LEN	SCREEN_WIDTH
+#define	LS_MSG_MAX_BYTE	(LS_MSG_MAX_LEN * CHAR_MAX_LEN_UTF_8)
+
 typedef struct {
-	char	msg[SCREEN_WIDTH * 2 + 1];
+	char	msg[LS_MSG_MAX_BYTE + 1];
 	curs_attr_t	attr;
 } ls_msg_t;
 
@@ -175,12 +179,21 @@ typedef struct {
 * ե졼ࡦ졼
 ***************************************************************/
 
+#define	TIMER_FRAME	4
+
 #define	ANIM_FRAME_N	((long)(1.0 * 60.0))
 
 #define	ANIM_FRAME_N_BLAST_STD	((long)(0.5 * 60.0))
 #define	ANIM_FRAME_N_BLAST_DESTRUCTION	((long)(2.0 * 60.0))
 #define	ANIM_FRAME_N_BLAST_NINJA_CRTCL	((long)(1.5 * 60.0))
 
+/***************************************************************
+* ӥ奢롦ե
+***************************************************************/
+
+/* ӥ奢롦եȤΥޥåפξõʸ */
+#define	MAP_VFX_NULL	'\0'
+
 /***************************************************************
 * ӥ奢롦եȤΥե륿
 ***************************************************************/
@@ -229,6 +242,117 @@ typedef unsigned long	flg_vfx_mnstr_t;
 /* ơ֥Υ */
 #define	KEY_TAB_MAX_N	512
 
+/***************************************************************
+* ե󥯥󡦥
+***************************************************************/
+
+/* ܥֹ椫饭ؤѴ */
+
+#define	KEY_N_ALL( fn )	(KEY_N_0 + (fn))
+#define	KEY_N_FUNC( fn )	(KEY_N_FUNC_0 + (fn))
+#define	KEY_N_MOUSE_CLICK( fn )	(KEY_N_MOUSE_CLICK_0 + (fn))
+#define	KEY_N_MOUSE_D_CLICK( fn )	(KEY_N_MOUSE_D_CLICK_0 + (fn))
+#define	KEY_N_MOUSE_P_CLICK( fn )	(KEY_N_MOUSE_P_CLICK_0 + (fn))
+#define	KEY_N_MOUSE_PRESS( fn )	(KEY_N_MOUSE_PRESS_0 + (fn))
+#define	KEY_N_MOUSE_RELEASE( fn )	(KEY_N_MOUSE_RELEASE_0 + (fn))
+#define	KEY_N_MOUSE_MOTION( fn )	(KEY_N_MOUSE_MOTION_0 + (fn))
+#define	KEY_N_JOYSTICK( fn )	(KEY_N_JOYSTICK_0 + (fn))
+
+/* ޥΥܥֹ */
+
+typedef enum {
+	KEY_N_MOUSE_0,
+	KEY_N_MOUSE_1,
+	KEY_N_MOUSE_2,
+	KEY_N_MOUSE_3,
+	KEY_N_MOUSE_4,
+	KEY_N_MOUSE_5,
+	KEY_N_MOUSE_6,
+	KEY_N_MOUSE_7,
+	KEY_N_MOUSE_8,
+	KEY_N_MOUSE_9,
+
+	KEY_N_MOUSE_L = KEY_N_MOUSE_0,
+	KEY_N_MOUSE_M = KEY_N_MOUSE_1,
+	KEY_N_MOUSE_R = KEY_N_MOUSE_2,
+
+	KEY_N_MOUSE_PRESS_L_CLICK_R = KEY_N_MOUSE_0,
+	KEY_N_MOUSE_PRESS_R_CLICK_L = KEY_N_MOUSE_1,
+
+	KEY_N_MOUSE_MOTION_LEFT = KEY_N_MOUSE_0,
+	KEY_N_MOUSE_MOTION_RIGHT = KEY_N_MOUSE_1,
+	KEY_N_MOUSE_MOTION_UP = KEY_N_MOUSE_2,
+	KEY_N_MOUSE_MOTION_DOWN = KEY_N_MOUSE_3,
+} key_n_mouse_t;
+
+/* ե󥯥󡦥󥹤ϰϡͽΰޤ */
+
+typedef enum {
+	KEY_N_0 = 0x1000,
+
+	KEY_N_FUNC_0 = 0x1000,
+	KEY_N_FUNC_MAX = 0x10ff,
+
+	KEY_N_MOUSE_CLICK_0 = 0x1100,
+	KEY_N_MOUSE_CLICK_MAX = 0x11ff,
+
+	KEY_N_MOUSE_D_CLICK_0 = 0x1200,
+	KEY_N_MOUSE_D_CLICK_MAX = 0x12ff,
+
+	KEY_N_MOUSE_P_CLICK_0 = 0x1300,
+	KEY_N_MOUSE_P_CLICK_MAX = 0x13ff,
+
+	KEY_N_MOUSE_PRESS_0 = 0x1400,
+	KEY_N_MOUSE_PRESS_MAX = 0x14ff,
+
+	KEY_N_MOUSE_RELEASE_0 = 0x1500,
+	KEY_N_MOUSE_RELEASE_MAX = 0x15ff,
+
+	KEY_N_MOUSE_MOTION_0 = 0x1600,
+	KEY_N_MOUSE_MOTION_MAX = 0x16ff,
+
+	KEY_N_JOYSTICK_0 = 0x1700,
+	KEY_N_JOYSTICK_MAX = 0x17ff,
+
+	KEY_N_MAX = 0x1fff,
+} key_n_t;
+
+/* ե󥯥󡦥󥹤λѤƤ륭ο */
+
+#define	KEY_N_FUNC_MAX_N	48
+#define	KEY_N_MOUSE_CLICK_MAX_N	10
+#define	KEY_N_MOUSE_D_CLICK_MAX_N	10
+#define	KEY_N_MOUSE_P_CLICK_MAX_N	10
+#define	KEY_N_MOUSE_PRESS_MAX_N	10
+#define	KEY_N_MOUSE_RELEASE_MAX_N	10
+#define	KEY_N_MOUSE_MOTION_MAX_N	10
+#define	KEY_N_JOYSTICK_MAX_N	20
+
+/* ƬΥǥå */
+
+#define	KEY_N_FUNC_HEAD_N	0
+#define	KEY_N_MOUSE_HEAD_N	KEY_N_FUNC_MAX_N
+#define	KEY_N_JOYSTICK_HEAD_N	(KEY_N_FUNC_MAX_N + KEY_N_MOUSE_MAX_N)
+
+/* ޥܥκ */
+
+#define	KEY_N_MOUSE_MAX_N	(0\
+	+ KEY_N_MOUSE_CLICK_MAX_N \
+	+ KEY_N_MOUSE_D_CLICK_MAX_N \
+	+ KEY_N_MOUSE_P_CLICK_MAX_N \
+	+ KEY_N_MOUSE_PRESS_MAX_N \
+	+ KEY_N_MOUSE_RELEASE_MAX_N \
+	+ KEY_N_MOUSE_MOTION_MAX_N \
+	)
+
+/* ե󥯥󡦥󥹤λѤƤ륭κ */
+
+#define	KEY_N_MAX_N	(0\
+	+ KEY_N_FUNC_MAX_N \
+	+ KEY_N_MOUSE_MAX_N \
+	+ KEY_N_JOYSTICK_MAX_N \
+	)
+
 /***************************************************************
 * ɽСǡμ
 ***************************************************************/
@@ -328,6 +452,34 @@ typedef enum {
 	DATA_N_BAR,
 	DATA_N_NIN,
 
+	DATA_N_SKILL_NAME_A,
+	DATA_N_SKILL_NAME_B,
+	DATA_N_SKILL_LEV,
+	DATA_N_SKILL_EXP,
+	DATA_N_SKILL_NEED_EXP,
+
+	DATA_N_SKILL_A_00,
+	DATA_N_SKILL_A_01,
+	DATA_N_SKILL_A_02,
+	DATA_N_SKILL_A_03,
+	DATA_N_SKILL_A_04,
+	DATA_N_SKILL_A_05,
+	DATA_N_SKILL_A_06,
+	DATA_N_SKILL_A_07,
+	DATA_N_SKILL_A_08,
+	DATA_N_SKILL_A_09,
+
+	DATA_N_SKILL_B_00,
+	DATA_N_SKILL_B_01,
+	DATA_N_SKILL_B_02,
+	DATA_N_SKILL_B_03,
+	DATA_N_SKILL_B_04,
+	DATA_N_SKILL_B_05,
+	DATA_N_SKILL_B_06,
+	DATA_N_SKILL_B_07,
+	DATA_N_SKILL_B_08,
+	DATA_N_SKILL_B_09,
+
 	DATA_N_ITEM_00,
 	DATA_N_ITEM_01,
 	DATA_N_ITEM_02,
@@ -359,6 +511,7 @@ typedef enum {
 	DATA_N_EQUIP_FEET,
 	DATA_N_EQUIP_LIGHT_SOURCE,
 
+	DATA_N_SWAP_PAGE_MBR,
 	DATA_N_PREV_PAGE,
 	DATA_N_NEXT_PAGE,
 
@@ -404,5 +557,42 @@ typedef enum {
 	EDIT_CLASS_MAX_N,
 } edit_class_t;
 
+/***************************************************************
+* Խ륹롦ǡμ
+***************************************************************/
+
+typedef enum {
+	EDIT_SKILL_NULL,
+
+	EDIT_SKILL_A_00,
+	EDIT_SKILL_A_01,
+	EDIT_SKILL_A_02,
+	EDIT_SKILL_A_03,
+	EDIT_SKILL_A_04,
+	EDIT_SKILL_A_05,
+	EDIT_SKILL_A_06,
+	EDIT_SKILL_A_07,
+	EDIT_SKILL_A_08,
+	EDIT_SKILL_A_09,
+
+	EDIT_SKILL_CLASS_NAME,
+
+	EDIT_SKILL_B_00,
+	EDIT_SKILL_B_01,
+	EDIT_SKILL_B_02,
+	EDIT_SKILL_B_03,
+	EDIT_SKILL_B_04,
+	EDIT_SKILL_B_05,
+	EDIT_SKILL_B_06,
+	EDIT_SKILL_B_07,
+	EDIT_SKILL_B_08,
+	EDIT_SKILL_B_09,
+
+	EDIT_SKILL_PREV,
+	EDIT_SKILL_NEXT,
+
+	EDIT_SKILL_MAX_N,
+} edit_skill_t;
+
 #endif	/* DRAW_H */
 
