@@ -17,6 +17,7 @@ crsr_ptn_t	*get_sub_crsr_ptn( void );
 bool_t	set_crsr_ptn( crsr_ptn_t *dst, crsr_ptn_t *src );
 void	up_dun_town( void );
 void	up_dun( long d );
+void	up_dun_msg( void );
 void	enter_last_boss_dun( void );
 void	reset_all_save( bool_t flg_appear );
 void	reset_all( bool_t flg_appear );
@@ -70,6 +71,9 @@ void	set_map_total_crsr_attr(
 );
 void	set_map_total_obj( long x, long y, long dx, long dy );
 void	set_map_total_square( long x, long y, long dx, long dy );
+void	set_map_total_square_n(
+	long mbr_n, long x, long y, long dx, long dy
+);
 void	set_map_total_chr( long x, long y, long dx, long dy );
 void	set_map_total_chr_1( long x, long y );
 void	set_map_total_crsr_ptn(
@@ -135,6 +139,7 @@ char	get_chr_mnr_crsr( void );
 char	get_chr_crsr( bool_t flg_mjr );
 long	chk_nest_flg_dun( void );
 bool_t	save_dun( void );
+void	check_memory_dun_c( void );
 
 #ifdef __cplusplus
 }
