@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2009 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: dun.c,v 1.129 2008/10/07 10:27:00 bowkenken Exp $
+* $Id: dun.c,v 1.155 2009/08/21 11:34:13 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -49,6 +49,9 @@
 /* 곬ʤο */
 #define	STAIRS_DOWN_MAX_N	2
 
+/* 饹ȡơγʤο */
+#define	STAIRS_LAST_BOSS_MAX_N	2
+
 /* 겼ȽγʤޤǤεΥ */
 #define	STAIRS_MAX_R	2
 
@@ -93,16 +96,20 @@
 ***************************************************************/
 
 static dun_t	dun;
-
-static long	g_last_boss_lev;
+check_memory_def( check_memory_dun_c_dun )
 
 static area_t	g_room_ptn_max_n, g_boss_room_ptn_max_n;
+check_memory_def( check_memory_dun_c_g_room_ptn_max_n )
 
 static crsr_ptn_t	g_crsr_ptn, g_crsr_ptn_usr[CRSR_PTN_MAX_N];
+check_memory_def( check_memory_dun_c_g_crsr_ptn )
 static crsr_ptn_t	g_sub_crsr_ptn;
+check_memory_def( check_memory_dun_c_g_sub_crsr_ptn )
 static long	g_crsr_ptn_n, g_crsr_ptn_max_n;
+check_memory_def( check_memory_dun_c_g_crsr_ptn_n )
 
 static long	g_nest_flg_dun;
+check_memory_def( check_memory_dun_c_g_nest_flg_dun )
 
 /***************************************************************
 * ǡե
@@ -122,8 +129,6 @@ void	init_dun( void )
 	g_nest_flg_dun = make_nest_flg();
 	bgn_nest_flg( g_nest_flg_dun );
 
-	g_last_boss_lev = -DUN_LEV_LOOP;
-
 	g_room_ptn_max_n = sizeof( room_ptn )
 			/ sizeof( room_ptn_t );
 	g_boss_room_ptn_max_n = sizeof( boss_room_ptn )
@@ -250,7 +255,8 @@ void	reset_dun( bool_t flg_wall )
 			} else {
 				dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
 				dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
-				dun.map.obj.flg[y][x] = FLG_MAP_OBJ_PASS;
+				dun.map.obj.flg[y][x] = (FLG_MAP_OBJ_PASS
+						| FLG_MAP_OBJ_LOOK);
 			}
 
 			dun.map.chr.mjr[y][x] = FACE_MJR_NULL;
@@ -369,8 +375,6 @@ void	up_dun( long d )
 	if( dun.lev < -DUN_MAX_LEV_BASE )
 		dun.lev = -DUN_MAX_LEV_BASE;
 
-	set_last_boss_lev( dun.lev );
-
 	if( g_flg_init ){
 		if( get_flg_no_save() )
 			menu_save_n( TRUE, SAVE_N_AUTO );
@@ -378,18 +382,7 @@ void	up_dun( long d )
 			menu_save_n( FALSE, SAVE_N_AUTO );
 	}
 
-	if( sgn_l( dun.lev ) >= +1 ){
-		print_msg( FLG_NULL, MSG_UP_DOWN_DUN_FLOOR,
-				labs( dun.lev ) );
-		change_scene( SCENE_N_DUN );
-	} else if( sgn_l( dun.lev ) <= -1 ){
-		print_msg( FLG_NULL, MSG_UP_DOWN_DUN_BASE,
-				labs( dun.lev ) );
-		change_scene( SCENE_N_DUN );
-	} else {
-		print_msg( FLG_NULL, MSG_UP_DOWN_DUN_GROUND );
-		change_scene( SCENE_N_TOWN );
-	}
+	up_dun_msg();
 
 	/* Сãؤ򹹿 */
 
@@ -415,6 +408,26 @@ void	up_dun( long d )
 	reset_all( TRUE );
 }
 
+/***************************************************************
+* µܤ겼ꤷݤΥåɽȥڤؤ
+***************************************************************/
+
+void	up_dun_msg( void )
+{
+	if( sgn_l( dun.lev ) >= +1 ){
+		print_msg( FLG_NULL, MSG_UP_DOWN_DUN_FLOOR,
+				labs( dun.lev ) );
+		change_scene( SCENE_N_DUN );
+	} else if( sgn_l( dun.lev ) <= -1 ){
+		print_msg( FLG_NULL, MSG_UP_DOWN_DUN_BASE,
+				labs( dun.lev ) );
+		change_scene( SCENE_N_DUN );
+	} else {
+		print_msg( FLG_NULL, MSG_UP_DOWN_DUN_GROUND );
+		change_scene( SCENE_N_TOWN );
+	}
+}
+
 /***************************************************************
 * 饹ܥγؤ
 ***************************************************************/
@@ -422,11 +435,6 @@ void	up_dun( long d )
 void	enter_last_boss_dun( void )
 {
 	change_last_scene( LAST_SCENE_N_ENTER );
-
-	if( dun.lev != DUN_LEV_LAST_BOSS )
-		set_last_boss_lev( dun.lev );
-	dun.lev = DUN_LEV_LAST_BOSS;
-
 	reset_all( TRUE );
 	set_map_event_tab( last_boss_map_event_tab );
 }
@@ -440,7 +448,7 @@ void	reset_all_save( bool_t flg_appear )
 {
 	srandm( g_randm_seed_save );
 
-	reset_all( flg_appear );
+	reset_all( TRUE );
 }
 
 /***************************************************************
@@ -466,7 +474,7 @@ void	reset_all( bool_t flg_appear )
 	clr_map_all_mbr();
 	clr_map_all_pet();
 
-	if( dun.lev == DUN_LEV_LAST_BOSS ){
+	if( get_scene() == SCENE_N_LAST_BOSS ){
 		make_dun_last_boss();
 		appear_party_last_boss();
 	} else if( dun.lev == 0 ){
@@ -584,7 +592,8 @@ void	make_dun( void )
 	/* 饹ܥؤγʤ¤ */
 
 	if( labs( dun.lev ) >= labs( DUN_LEV_LAST_BOSS_STAIRS ) )
-		make_stairs_randm( FACE_MNR_STAIRS_LAST_BOSS );
+		for( i = 0; i < STAIRS_LAST_BOSS_MAX_N; i++ )
+			make_stairs_randm( FACE_MNR_STAIRS_LAST_BOSS );
 
 	/* ꥢƥ */
 
@@ -597,7 +606,8 @@ void	make_dun( void )
 
 			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
 			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
-			dun.map.obj.flg[y][x] = FLG_MAP_OBJ_PASS;
+			dun.map.obj.flg[y][x] = (FLG_MAP_OBJ_PASS
+					| FLG_MAP_OBJ_LOOK);
 			if( make_item( x, y, dun.lev )
 					!= ITEM_KIND_NULL ){
 				item_n++;
@@ -614,7 +624,8 @@ void	make_dun( void )
 
 			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
 			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
-			dun.map.obj.flg[y][x] |= FLG_MAP_OBJ_PASS;
+			dun.map.obj.flg[y][x] |= (FLG_MAP_OBJ_PASS
+					| FLG_MAP_OBJ_LOOK);
 			if( rate_randm( MIMIC_RATE ) ){
 				make_mnstr( x, y, TRUE, dun.lev,
 						MNSTR_KIND_MIMIC );
@@ -650,7 +661,8 @@ void	make_dun( void )
 
 			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
 			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
-			dun.map.obj.flg[y][x] |= FLG_MAP_OBJ_PASS;
+			dun.map.obj.flg[y][x] |= (FLG_MAP_OBJ_PASS
+					| FLG_MAP_OBJ_LOOK);
 
 			mnstr_kind = get_mnstr_kind_from_face( mjr, mnr );
 			if( mnstr_kind == MNSTR_KIND_NULL )
@@ -669,7 +681,8 @@ void	make_dun( void )
 
 			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
 			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
-			dun.map.obj.flg[y][x] |= FLG_MAP_OBJ_PASS;
+			dun.map.obj.flg[y][x] |= (FLG_MAP_OBJ_PASS
+					| FLG_MAP_OBJ_LOOK);
 
 			make_trap( x, y, dun.lev );
 		}
@@ -773,19 +786,6 @@ void	make_dun( void )
 	end_nest_flg( g_nest_flg_dun );
 }
 
-/***************************************************************
-* 饹ܥΥ٥γؤ
-* long dun_lev : 
-***************************************************************/
-
-void	set_last_boss_lev( long dun_lev )
-{
-	if( dun_lev == 0 )
-		g_last_boss_lev = -DUN_LEV_LOOP;
-	else
-		g_last_boss_lev = dun_lev;
-}
-
 /***************************************************************
 * 饹ܥγؤ
 ***************************************************************/
@@ -804,11 +804,14 @@ void	make_dun_last_boss( void )
 	/* ¤ */
 
 	x = AREA_MAP_MAX_X / 2;
-	y = 0;
+	y = AREA_MAP_MAX_Y - LAST_BOSS_ROOM_PTN_MAX_N;
+	if( y < 0 )
+		y = 0;
+
 	for( i = 0; i < LAST_BOSS_ROOM_PTN_MAX_N; i++ ){
-		y = 0 + i;
 		make_area_last_boss( &(last_boss_room_ptn[i]),
 				(area_t)1, x, y );
+		y++;
 	}
 
 	/* ޥåפüɤˤ */
@@ -831,14 +834,14 @@ void	make_dun_last_boss( void )
 
 			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
 			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
-			dun.map.obj.flg[y][x] |= FLG_MAP_OBJ_PASS;
+			dun.map.obj.flg[y][x] |= (FLG_MAP_OBJ_PASS
+					| FLG_MAP_OBJ_LOOK);
 
 			mnstr_kind = get_mnstr_kind_from_face( mjr, mnr );
 			if( mnstr_kind == MNSTR_KIND_NULL )
 				continue;
 
-			make_mnstr( x, y, TRUE, g_last_boss_lev,
-					mnstr_kind );
+			make_mnstr( x, y, TRUE, dun.lev, mnstr_kind );
 		}
 	}
 
@@ -861,7 +864,8 @@ void	make_fence( void )
 
 		dun.map.obj.mjr[n][0] = FACE_MJR_WALL;
 		dun.map.obj.mnr[n][0] = FACE_MNR_WALL;
-		dun.map.obj.flg[n][0] &= ~(FLG_MAP_OBJ_PASS);
+		dun.map.obj.flg[n][0] &= ~(FLG_MAP_OBJ_PASS
+				| FLG_MAP_OBJ_LOOK);
 	}
 	for( n = 0; n < MAP_MAX_Y; n++ ){
 		if( dun.map.obj.mnr[n][MAP_MAX_X - 1] == FACE_MNR_GATE )
@@ -869,7 +873,8 @@ void	make_fence( void )
 
 		dun.map.obj.mjr[n][MAP_MAX_X - 1] = FACE_MJR_WALL;
 		dun.map.obj.mnr[n][MAP_MAX_X - 1] = FACE_MNR_WALL;
-		dun.map.obj.flg[n][MAP_MAX_X - 1] &= ~(FLG_MAP_OBJ_PASS);
+		dun.map.obj.flg[n][MAP_MAX_X - 1] &= ~(FLG_MAP_OBJ_PASS
+				| FLG_MAP_OBJ_LOOK);
 	}
 
 	for( n = 0; n < MAP_MAX_X; n++ ){
@@ -878,7 +883,8 @@ void	make_fence( void )
 
 		dun.map.obj.mjr[0][n] = FACE_MJR_WALL;
 		dun.map.obj.mnr[0][n] = FACE_MNR_WALL;
-		dun.map.obj.flg[0][n] &= ~(FLG_MAP_OBJ_PASS);
+		dun.map.obj.flg[0][n] &= ~(FLG_MAP_OBJ_PASS
+				| FLG_MAP_OBJ_LOOK);
 	}
 	for( n = 0; n < MAP_MAX_X; n++ ){
 		if( dun.map.obj.mnr[MAP_MAX_Y - 1][n] == FACE_MNR_GATE )
@@ -886,7 +892,8 @@ void	make_fence( void )
 
 		dun.map.obj.mjr[MAP_MAX_Y - 1][n] = FACE_MJR_WALL;
 		dun.map.obj.mnr[MAP_MAX_Y - 1][n] = FACE_MNR_WALL;
-		dun.map.obj.flg[MAP_MAX_Y - 1][n] &= ~(FLG_MAP_OBJ_PASS);
+		dun.map.obj.flg[MAP_MAX_Y - 1][n] &= ~(FLG_MAP_OBJ_PASS
+				| FLG_MAP_OBJ_LOOK);
 	}
 }
 
@@ -1047,16 +1054,20 @@ void	set_room( room_ptn_t *room, area_t n, long x, long y, long flg_rev )
 					flg &= ~(FLG_MAP_OBJ_PASS);
 				else
 					flg |= FLG_MAP_OBJ_PASS;
+				flg |= FLG_MAP_OBJ_LOOK;
 				break;
 			case FACE_MJR_DOOR_OPEN:
-				flg |= FLG_MAP_OBJ_PASS;
+				flg |= (FLG_MAP_OBJ_PASS
+						| FLG_MAP_OBJ_LOOK);
 				break;
 			case FACE_MJR_WALL:
 			case FACE_MJR_DOOR_CLOSE:
-				flg &= ~(FLG_MAP_OBJ_PASS);
+				flg &= ~(FLG_MAP_OBJ_PASS
+						| FLG_MAP_OBJ_LOOK);
 				break;
 			default:
-				flg |= FLG_MAP_OBJ_PASS;
+				flg |= (FLG_MAP_OBJ_PASS
+						| FLG_MAP_OBJ_LOOK);
 			}
 			dun.map.obj.flg[my][mx] = flg;
 
@@ -1100,7 +1111,7 @@ bool_t	chk_lighted_room( void )
 {
 	rate_t	rate;
 
-	if( dun.lev == DUN_LEV_LAST_BOSS )
+	if( get_scene() == SCENE_N_LAST_BOSS )
 		return TRUE;
 
 	rate = LIGHTED_ROOM_MAX_RATE - (randm( labs( dun.lev ) ) * 2);
@@ -1665,7 +1676,11 @@ door_t	*make_door( long bx, long by, long ex, long ey, char mnr )
 
 	flg = FLG_NULL;
 
-	if( mnr == FACE_MNR_NULL ){
+	if( get_scene() == SCENE_N_LAST_BOSS ){
+		flg |= FLG_DOOR_SECRET;
+		if( mnr == FACE_MNR_GATE )
+			flg |= FLG_DOOR_GATE;
+	} else if( mnr == FACE_MNR_NULL ){
 		if( chk_rate_door_open( dun.lev ) )
 			flg |= FLG_DOOR_OPEN;
 
@@ -1800,20 +1815,37 @@ door_t	*make_door_flg(
 	if( dy <= 0 )
 		return NULL;
 
-	/* ΰ֤ˤǤ˥ɥʤ return */
+	/* ΰ֤ˤǤ礭ɥʤ return */
 	for( i = 0; i < n; i++ ){
-		if( dun.door[i].x != bx )
+		if( bx < dun.door[i].x )
 			continue;
-		if( dun.door[i].y != by )
+		if( by < dun.door[i].y )
 			continue;
-		if( dun.door[i].dx != dx )
+		if( (dun.door[i].x + dun.door[i].dx) < (bx + dx) )
 			continue;
-		if( dun.door[i].dy != dy )
+		if( (dun.door[i].y + dun.door[i].dy) < (by + dy) )
 			continue;
 
 		return NULL;
 	}
 
+	/* ΰ֤ˤǤ˾ɥʤ */
+	for( i = 0; i < n; i++ ){
+		if( dun.door[i].x < bx )
+			continue;
+		if( dun.door[i].y < by )
+			continue;
+		if( (bx + dx) < (dun.door[i].x + dun.door[i].dx) )
+			continue;
+		if( (by + dy) < (dun.door[i].y + dun.door[i].dy) )
+			continue;
+
+		dun.door[i].x = MAP_DEL_X;
+		dun.door[i].y = MAP_DEL_Y;
+		dun.door[i].dx = 0;
+		dun.door[i].dy = 0;
+	}
+
 	dun.door[n].n = n;
 	dun.door[n].x = bx;
 	dun.door[n].y = by;
@@ -1851,7 +1883,7 @@ void	set_face_door( long dr_n )
 
 	if( chk_flg( p->flg, FLG_DOOR_OPEN ) ){
 		mjr = FACE_MJR_DOOR_OPEN;
-		flg |= FLG_MAP_OBJ_PASS;
+		flg |= (FLG_MAP_OBJ_PASS | FLG_MAP_OBJ_LOOK);
 	}
 
 	if( chk_flg( p->flg, FLG_DOOR_JAMMED ) )
@@ -1866,7 +1898,7 @@ void	set_face_door( long dr_n )
 		p->flg |= FLG_DOOR_OPEN;
 		mjr = FACE_MJR_DOOR_OPEN;
 		mnr = FACE_MNR_DOOR_BROKEN;
-		flg |= FLG_MAP_OBJ_PASS;
+		flg |= (FLG_MAP_OBJ_PASS | FLG_MAP_OBJ_LOOK);;
 	}
 
 	if( chk_face_window( p->x, p->y,
@@ -1878,7 +1910,7 @@ void	set_face_door( long dr_n )
 		p->flg &= ~(FLG_DOOR_OPEN);
 		mjr = FACE_MJR_DOOR_CLOSE;
 		mnr = FACE_MNR_DOOR_SECRET;
-		flg &= ~FLG_MAP_OBJ_PASS;
+		flg &= ~(FLG_MAP_OBJ_PASS | FLG_MAP_OBJ_LOOK);
 		flg |= FLG_MAP_OBJ_LOOK_WALL;
 	}
 
@@ -1886,7 +1918,7 @@ void	set_face_door( long dr_n )
 		mjr = FACE_MJR_DOOR_CLOSE;
 		/* mnr  make_door() Ѥ */
 		mnr = p->mnr;
-		flg &= ~FLG_MAP_OBJ_PASS;
+		flg &= ~(FLG_MAP_OBJ_PASS | FLG_MAP_OBJ_LOOK);
 		flg |= FLG_MAP_OBJ_FIND;
 	}
 
@@ -1898,7 +1930,7 @@ void	set_face_door( long dr_n )
 	if( chk_flg( p->flg, FLG_DOOR_GATE ) ){
 		mjr = FACE_MJR_DOOR_CLOSE;
 		mnr = FACE_MNR_GATE;
-		flg &= ~FLG_MAP_OBJ_PASS;
+		flg &= ~(FLG_MAP_OBJ_PASS | FLG_MAP_OBJ_LOOK);
 		flg |= FLG_MAP_OBJ_FIND;
 	}
 
@@ -2197,20 +2229,56 @@ void	make_stairs_randm( char face )
 }
 
 /***************************************************************
-* ɸꤷơʤ
+* 濴κɸꤷơõƤ鳬ʤ
 * long x : X ɸ
 * long y : Y ɸ
 * char face : ʤμδʸ
 ***************************************************************/
 
 void	make_stairs( long x, long y, char face )
+{
+	const long	max_r = MAP_MAX_X;
+	long	r;
+
+	for( r = 0; r < max_r; r++ ){
+		long	dx, dy;
+
+		for( dx = -r, dy = -r; dx <= +r; dx++ )
+			if( make_stairs_sub( x + dx, y + dy, face ) )
+				return;
+
+		for( dx = +r, dy = -r; dy <= +r; dy++ )
+			if( make_stairs_sub( x + dx, y + dy, face ) )
+				return;
+
+		for( dx = +r, dy = +r; dx >= -r; dx-- )
+			if( make_stairs_sub( x + dx, y + dy, face ) )
+				return;
+
+		for( dx = -r, dy = +r; dy >= -r; dy-- )
+			if( make_stairs_sub( x + dx, y + dy, face ) )
+				return;
+	}
+}
+
+/***************************************************************
+* ɸꤷơʤ
+* long x : X ɸ
+* long y : Y ɸ
+* char face : ʤμδʸ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	make_stairs_sub( long x, long y, char face )
 {
 	char	mjr, mnr;
 
+	if( !clip_pos( x, y ) )
+		return FALSE;
 	if( dun.map.obj.mjr[y][x] != FACE_MJR_FLOOR )
-		return;
+		return FALSE;
 	if( dun.map.obj.mnr[y][x] != FACE_MNR_FLOOR )
-		return;
+		return FALSE;
 
 	if( face == FACE_MNR_STAIRS_LAST_BOSS ){
 		mjr = FACE_MJR_STAIRS_LAST_BOSS;
@@ -2218,13 +2286,13 @@ void	make_stairs( long x, long y, char face )
 	} else if( (face == FACE_MJR_STAIRS_DOWN)
 			|| (face == FACE_MNR_STAIRS_DOWN) ){
 		if( labs( dun.lev ) >= labs( DUN_MAX_LEV_BASE ) )
-			return;
+			return TRUE;
 
 		mjr = FACE_MJR_STAIRS_DOWN;
 		mnr = FACE_MNR_STAIRS_DOWN;
 	} else {
 		if( labs( dun.lev ) >= labs( DUN_MAX_LEV_FLOOR ) )
-			return;
+			return TRUE;
 
 		mjr = FACE_MJR_STAIRS_UP;
 		mnr = FACE_MNR_STAIRS_UP;
@@ -2232,6 +2300,8 @@ void	make_stairs( long x, long y, char face )
 
 	dun.map.obj.mjr[y][x] = mjr;
 	dun.map.obj.mnr[y][x] = mnr;
+
+	return TRUE;
 }
 
 /***************************************************************
@@ -2281,6 +2351,8 @@ void	set_map_total( long x, long y, long dx, long dy )
 	crsr_ptn_t	*sub_ptn = get_sub_crsr_ptn();
 
 	/* ط */
+	set_map_total_water( x, y, dx, dy );
+	set_map_total_last_boss_bg( x, y, dx, dy );
 	set_map_total_bg( x, y, dx, dy );
 
 	/* ° */
@@ -2295,12 +2367,69 @@ void	set_map_total( long x, long y, long dx, long dy )
 
 	/* 󥹥ȥС */
 	set_map_total_chr( x, y, dx, dy );
+	set_map_total_last_boss_fg( x, y, dx, dy );
 
 	/* Υѥ */
 	set_map_total_crsr_ptn( sub_crsr, sub_ptn, x, y, dx, dy );
 	set_map_total_crsr_ptn( crsr, ptn, x, y, dx, dy );
 }
 
+/***************************************************************
+* طʥޥåפοϷ쥤䡼򹹿
+* long x : X ɸ
+* long y : Y ɸ
+* long dx : 
+* long dy : ⤵
+***************************************************************/
+
+void	set_map_total_water( long x, long y, long dx, long dy )
+{
+	long	bx, by;
+	long	ex, ey;
+	long	xx, yy;
+	char	mjr, mnr;
+	curs_attr_n_t	attr_n_water, attr_n_lava;
+	curs_attr_t	*attr_dflt;
+
+	attr_n_water = CURS_ATTR_N_MAP_NORMAL;
+	attr_n_lava = CURS_ATTR_N_MAP_MNSTR;
+	attr_dflt = get_curs_attr();
+
+	bx = x;
+	by = y;
+	ex = x + dx - 1;
+	ey = y + dy - 1;
+	if( bx < 0 )
+		bx = 0;
+	if( by < 0 )
+		by = 0;
+	if( ex > MAP_MAX_X - 1 )
+		ex = MAP_MAX_X - 1;
+	if( ey > MAP_MAX_Y - 1 )
+		ey = MAP_MAX_Y - 1;
+
+	for( yy = by; yy <= ey; yy++ ){
+		for( xx = bx; xx <= ex; xx++ ){
+			set_face_from_map_obj(
+					TRUE, xx, yy, &mjr, &mnr,
+					FALSE );
+			if( mjr != FACE_MJR_WATER )
+				continue;
+
+			dun.map.total.mjr[yy][xx] = mjr;
+			dun.map.total.mnr[yy][xx] = mnr;
+			dun.map.total.flg[yy][xx] = FLG_NULL;
+			if( mnr == FACE_MNR_LAVA ){
+				dun.map.attr[yy][xx] = attr_dflt
+						[attr_n_lava];
+			} else {
+				dun.map.attr[yy][xx] = attr_dflt
+						[attr_n_water];
+			}
+		}
+	}
+}
+
 /***************************************************************
 * طʥޥåפ쥤䡼򹹿
 * long x : X ɸ
@@ -2339,6 +2468,8 @@ void	set_map_total_bg( long x, long y, long dx, long dy )
 			set_face_from_map_obj(
 					TRUE, xx, yy, &mjr, &mnr,
 					FALSE );
+			if( mjr == FACE_MJR_WATER )
+				continue;
 
 			dun.map.total.mjr[yy][xx] = mjr;
 			dun.map.total.mnr[yy][xx] = mnr;
@@ -2392,6 +2523,10 @@ void	set_face_from_map_obj(
 		*mjr = FACE_MJR_FLOOR;
 		*mnr = FACE_MNR_FLOOR;
 	}
+
+	if( (*mjr == FACE_MJR_FLOOR) && isalnum( *mnr ) )
+		*mnr = FACE_MNR_NULL;
+
 	if( flg_chk_find ){
 		if( (*mjr == FACE_MJR_FLOOR) && (dep <= 0) ){
 			*mjr = FACE_MJR_NULL;
@@ -2505,6 +2640,8 @@ void	set_map_total_obj( long x, long y, long dx, long dy )
 				continue;
 			if( mjr == FACE_MJR_WALL )
 				continue;
+			if( mjr == FACE_MJR_WATER )
+				continue;
 			if( chk_flg( flg, FLG_MAP_OBJ_LOOK_WALL ) )
 				continue;
 			if( chk_flg( flg, FLG_MAP_OBJ_LOOK_FLOOR ) )
@@ -2542,53 +2679,50 @@ void	set_map_total_obj( long x, long y, long dx, long dy )
 
 void	set_map_total_square( long x, long y, long dx, long dy )
 {
-	long	i, j;
-	long	xx, yy;
-	curs_attr_t	*attr_dflt;
+	long	i;
+
+	for( i = 0; i < MBR_MAX_N; i++ )
+		set_map_total_square_n( i, x, y, dx, dy );
+}
+
+/***************************************************************
+* 󥫡Υޥåפ쥤䡼򹹿
+* long mbr_n : Сֹ
+* long x : X ɸ
+* long y : Y ɸ
+* long dx : 
+* long dy : ⤵
+***************************************************************/
+
+void	set_map_total_square_n(
+	long mbr_n, long x, long y, long dx, long dy
+)
+{
+	long	sq_x, sq_y;
 	party_t	*pty;
+	curs_attr_t	*attr_dflt;
+
+	get_square_mbr_map_pos( mbr_n, &sq_x, &sq_y );
+	if( !clip_pos( sq_x, sq_y ) )
+		return;
+	if( sq_x < x )
+		return;
+	if( sq_y < y )
+		return;
+	if( sq_x > (x + dx - 1) )
+		return;
+	if( sq_y > (y + dy - 1) )
+		return;
 
 	pty = get_party();
-
-	if( !clip_pos( pty->square.x, pty->square.y ) )
-		return;
-	if( (pty->square.x + 1) < x )
-		return;
-	if( (pty->square.y + 1) < y )
-		return;
-	if( (pty->square.x - 1) > (x + dx - 1) )
-		return;
-	if( (pty->square.y - 1) > (y + dy - 1) )
-		return;
+	dun.map.total.mjr[sq_y][sq_x] = FACE_MJR_SQUARE;
+	if( chk_flg( pty->mbr[mbr_n]->stat, FLG_STAT_NOT_EXIST ) )
+		dun.map.total.mnr[sq_y][sq_x] = FACE_MNR_NULL;
+	else
+		dun.map.total.mnr[sq_y][sq_x] = pty->mbr[mbr_n]->face.mnr;
 
 	attr_dflt = get_curs_attr();
-
-	for( i = -1; i <= +1; i++ ){
-		yy = pty->square.y + i;
-		if( !clip_y( yy ) )
-			continue;
-
-		for( j = -1; j <= +1; j++ ){
-			mbr_t	*mbr;
-
-			xx = pty->square.x + j;
-			if( !clip_x( xx ) )
-				continue;
-
-			if( pty->square.chr_dir[i + 1][j + 1] == MBR_NO_POS )
-				continue;
-
-			dun.map.total.mjr[yy][xx] = FACE_MJR_SQUARE;
-			dun.map.attr[yy][xx]
-					= attr_dflt[CURS_ATTR_N_MAP_SQUARE];
-
-			mbr = pty->mbr[pty->square.chr_dir[i + 1][j + 1]];
-			if( chk_flg( mbr->stat, FLG_STAT_NOT_EXIST ) ){
-				continue;
-			}
-
-			dun.map.total.mnr[yy][xx] = mbr->face.mnr;
-		}
-	}
+	dun.map.attr[sq_y][sq_x] = attr_dflt[CURS_ATTR_N_MAP_SQUARE];
 }
 
 /***************************************************************
@@ -2604,10 +2738,6 @@ void	set_map_total_chr( long x, long y, long dx, long dy )
 	long	bx, by;
 	long	ex, ey;
 	long	xx, yy;
-	char	mjr, mnr;
-	flg_map_t	flg, flg_total;
-	chr_t	*chr_p;
-	curs_attr_n_t	attr_n;
 	curs_attr_t	*attr_dflt;
 
 	attr_dflt = get_curs_attr();
@@ -2627,64 +2757,82 @@ void	set_map_total_chr( long x, long y, long dx, long dy )
 
 	for( yy = by; yy <= ey; yy++ ){
 		for( xx = bx; xx <= ex; xx++ ){
-			bool_t	flg_rev_rev;
-
-			flg_total = dun.map.total.flg[yy][xx];
-
-			mjr = dun.map.chr.mjr[yy][xx];
-			mnr = dun.map.chr.mnr[yy][xx];
-			flg = dun.map.chr.flg[yy][xx];
-			chr_p = dun.map.chr_p[yy][xx];
-			attr_n = CURS_ATTR_N_NORMAL;
-			flg_rev_rev = FALSE;
-
-			if( mjr == FACE_MJR_NULL ){
-				continue;
-			} else if( chr_p == NULL ){
-				continue;
-			} else if( is_mbr( chr_p ) ){
-				mbr_t	*p;
-
-				/* Сä */
-				p = get_mbr( xx, yy );
-				if( p != NULL ){
-					attr_n = (curs_attr_n_t)
-							(CURS_ATTR_N_MBR_1
-							+ p->mbr_n);
-				}
-				/* Ʃʤ */
-				if( chk_flg( flg, FLG_MAP_CHR_INVISIBLE ) ){
-					flg_rev_rev = TRUE;
-				}
-			} else if( chr_p->kind == CHR_KIND_MNSTR ){
-				/* Ʃʤ */
-				if( chk_flg( flg, FLG_MAP_CHR_INVISIBLE ) )
-					continue;
-
-				/* 󥹥ä */
-				if( !chk_flg( flg, FLG_MAP_CHR_FIND ) ){
-					/* ĤäƤʤ */
-					continue;
-				}
-				if( chk_flg( flg, FLG_MAP_CHR_NPC ) ){
-					attr_n = CURS_ATTR_N_MAP_NPC;
-				} else {
-					attr_n = CURS_ATTR_N_MAP_MNSTR;
-				}
-			} else {
-				continue;
-			}
-
-			dun.map.total.mjr[yy][xx] = mjr;
-			dun.map.total.mnr[yy][xx] = mnr;
-			dun.map.total.flg[yy][xx] = flg_total;
-			dun.map.attr[yy][xx] = attr_dflt[attr_n];
-			if( flg_rev_rev )
-				dun.map.attr[yy][xx].attr ^= A_REVERSE;
+			set_map_total_chr_1( xx, yy );
 		}
 	}
 }
 
+/***************************************************************
+* 󥹥ȥСΥޥåפ쥤䡼򹹿 (1 )
+* long x : X ɸ
+* long y : Y ɸ
+***************************************************************/
+
+void	set_map_total_chr_1( long x, long y )
+{
+	char	mjr, mnr;
+	flg_map_t	flg, flg_total;
+	bool_t	flg_rev_rev;
+	chr_t	*chr_p;
+	curs_attr_n_t	attr_n;
+	curs_attr_t	*attr_dflt;
+
+	attr_dflt = get_curs_attr();
+
+	flg_total = dun.map.total.flg[y][x];
+
+	mjr = dun.map.chr.mjr[y][x];
+	mnr = dun.map.chr.mnr[y][x];
+	flg = dun.map.chr.flg[y][x];
+	chr_p = dun.map.chr_p[y][x];
+	attr_n = CURS_ATTR_N_NORMAL;
+	flg_rev_rev = FALSE;
+
+	if( mjr == FACE_MJR_NULL ){
+		return;
+	} else if( chr_p == NULL ){
+		return;
+	} else if( is_mbr( chr_p ) ){
+		mbr_t	*p;
+
+		/* Сä */
+
+		p = get_mbr( x, y );
+		if( p != NULL ){
+			attr_n = (curs_attr_n_t)
+					(CURS_ATTR_N_MBR_1 + p->mbr_n);
+		}
+		/* Ʃʤ */
+		if( chk_flg( flg, FLG_MAP_CHR_INVISIBLE ) ){
+			flg_rev_rev = TRUE;
+		}
+	} else if( chr_p->kind == CHR_KIND_MNSTR ){
+		/* 󥹥ä */
+
+		/* ĤäƤʤ */
+		if( !chk_flg( flg, FLG_MAP_CHR_FIND ) )
+			return;
+		/* Ʃʤ */
+		if( chk_flg( flg, FLG_MAP_CHR_INVISIBLE ) )
+			return;
+
+		if( chk_flg( flg, FLG_MAP_CHR_NPC ) ){
+			attr_n = CURS_ATTR_N_MAP_NPC;
+		} else {
+			attr_n = CURS_ATTR_N_MAP_MNSTR;
+		}
+	} else {
+		return;
+	}
+
+	dun.map.total.mjr[y][x] = mjr;
+	dun.map.total.mnr[y][x] = mnr;
+	dun.map.total.flg[y][x] = flg_total;
+	dun.map.attr[y][x] = attr_dflt[attr_n];
+	if( flg_rev_rev )
+		dun.map.attr[y][x].attr ^= A_REVERSE;
+}
+
 /***************************************************************
 * ΥѥΥޥåפ쥤䡼򹹿
 * long x : X ɸ
@@ -2827,7 +2975,8 @@ void	put_path( long x, long y, long dx, long dy, sect_t sect )
 
 			dun.map.obj.mjr[yy][xx] = FACE_MJR_FLOOR;
 			dun.map.obj.mnr[yy][xx] = FACE_MNR_FLOOR;
-			dun.map.obj.flg[yy][xx] |= FLG_MAP_OBJ_PASS;
+			dun.map.obj.flg[yy][xx] |= (FLG_MAP_OBJ_PASS
+					| FLG_MAP_OBJ_LOOK);
 		}
 	}
 }
@@ -3074,7 +3223,6 @@ void	find_obj_fill( long x, long y )
 	long	ax, ay;
 	long	sx, sy;
 	long	dx, dy;
-	char	mjr, mnr;
 	sect_t	sect;
 
 	ax = x / AREA_MAX_X * AREA_MAX_X;
@@ -3089,13 +3237,8 @@ void	find_obj_fill( long x, long y )
 			if( dun.map.sect[sy][sx] != sect )
 				continue;
 			if( !chk_flg( dun.map.obj.flg[sy][sx],
-					FLG_MAP_OBJ_PASS ) ){
-				mjr = dun.map.obj.mjr[sy][sx];
-				mnr = dun.map.obj.mnr[sy][sx];
-				if( mjr != FACE_MJR_FLOOR )
-					continue;
-				if( mnr != FACE_MNR_HOLE )
-					continue;
+					FLG_MAP_OBJ_LOOK ) ){
+				continue;
 			}
 
 			find_obj_fill_field( sx, sy );
@@ -3213,7 +3356,7 @@ void	turn_light_field(
 				continue;
 
 			if( !chk_flg( dun.map.obj.flg[sy][sx],
-					FLG_MAP_OBJ_PASS ) )
+					FLG_MAP_OBJ_LOOK ) )
 				continue;
 
 			if( chk_flg( dun.map.obj.flg[sy][sx],
@@ -3384,9 +3527,7 @@ bool_t	chk_find_pos( long x, long y )
 	if( !clip_pos( x, y ) )
 		return FALSE;
 
-	if( chk_flg( dun.map.obj.flg[y][x], FLG_MAP_OBJ_PASS ) )
-		return TRUE;
-	if( dun.map.obj.mjr[y][x] == FACE_MJR_FLOOR )
+	if( chk_flg( dun.map.obj.flg[y][x], FLG_MAP_OBJ_LOOK ) )
 		return TRUE;
 
 	return FALSE;
@@ -3424,13 +3565,27 @@ door_t	*get_door( long x, long y )
 	long	i;
 	door_t	*dr;
 
+	if( !clip_pos( x, y ) )
+		return NULL;
+
 	for( i = 0; i < dun.door_n; i++ ){
 		dr = &(dun.door[i]);
-		if( (dr->x <= x) && (x <= dr->x + dr->dx)
-				&& (dr->y <= y) && (y <= dr->y + dr->dy) ){
-			return dr;
-		}
+		if( x < dr->x )
+			continue;
+		if( x > dr->x + dr->dx - 1 )
+			continue;
+		if( y < dr->y )
+			continue;
+		if( y > dr->y + dr->dy - 1 )
+			continue;
+		if( dun.map.obj.mjr[y][x] != dr->mjr )
+			continue;
+		if( dun.map.obj.mnr[y][x] != dr->mnr )
+			continue;
+
+		return dr;
 	}
+
 	return NULL;
 }
 
@@ -3795,6 +3950,8 @@ bool_t	up_stairs( void )
 		return FALSE;
 	}
 
+	call_game_sound_play( SOUND_KIND_STAIRS_UP, 1 );
+
 	print_msg( FLG_NULL, MSG_UP_DUN );
 	up_dun( +1 );
 
@@ -3816,6 +3973,8 @@ bool_t	down_stairs( void )
 		return FALSE;
 	}
 
+	call_game_sound_play( SOUND_KIND_STAIRS_DOWN, 1 );
+
 	print_msg( FLG_NULL, MSG_DOWN_DUN );
 	up_dun( -1 );
 
@@ -3837,6 +3996,12 @@ bool_t	down_stairs_last_boss( void )
 		return FALSE;
 	}
 
+	if( !chk_enter_last_stage() ){
+		print_msg( FLG_NULL, MSG_ERR_STAIRS_DOWN_LAST_BOSS_1 );
+		print_msg( FLG_NULL, MSG_ERR_STAIRS_DOWN_LAST_BOSS_2 );
+		return FALSE;
+	}
+
 	print_msg( FLG_NULL, MSG_DOWN_DUN );
 	enter_last_boss_dun();
 
@@ -4064,8 +4229,8 @@ bool_t	clip_draw_x( long n )
 
 	if( g_flg_cui ){
 		draw = get_map_draw_pos();
-		if( (draw->x <= n)
-				&& (n <= (draw->x + WIN_MAP_WIDTH - 1)) ){
+		if( (n <= (draw->x + (get_map_col() / 2) - 1))
+				&& (draw->x <= n) ){
 			return TRUE;
 		}
 	}
@@ -4092,8 +4257,8 @@ bool_t	clip_draw_y( long n )
 
 	if( g_flg_cui ){
 		draw = get_map_draw_pos();
-		if( (draw->y <= n)
-				&& (n <= (draw->y + WIN_MAP_WIDTH - 1)) ){
+		if( (n <= (draw->y + get_map_row() - 1))
+				&& (draw->y <= n) ){
 			return TRUE;
 		}
 	}
@@ -4245,3 +4410,23 @@ bool_t	save_dun( void )
 
 	return TRUE;
 }
+
+/***************************************************************
+* ꡼å
+***************************************************************/
+
+void	check_memory_dun_c( void )
+{
+	check_memory( check_memory_dun_c_dun,
+			"dun.c: dun" );
+	check_memory( check_memory_dun_c_g_room_ptn_max_n,
+			"dun.c: g_room_ptn_max_n" );
+	check_memory( check_memory_dun_c_g_crsr_ptn,
+			"dun.c: g_crsr_ptn" );
+	check_memory( check_memory_dun_c_g_sub_crsr_ptn,
+			"dun.c: g_sub_crsr_ptn" );
+	check_memory( check_memory_dun_c_g_crsr_ptn_n,
+			"dun.c: g_crsr_ptn_n" );
+	check_memory( check_memory_dun_c_g_nest_flg_dun,
+			"dun.c: g_nest_flg_dun" );
+}
