@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: gf-conf.c,v 1.17 2009/05/12 01:30:40 bowkenken Exp $
+* $Id: gf-conf.c,v 1.25 2009/08/21 10:33:54 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -98,6 +98,11 @@ bool_t	load_game_conf( void )
 		break;
 	}
 
+	/* Сʥå */
+
+	if( chk_ver_old( &ver, 1, 0, 14 ) )
+		ver_up_game_conf_1_0_15();
+
 	/* ե򥯥 */
 
 	if( fclose( fp ) == EOF )
@@ -113,6 +118,24 @@ bool_t	load_game_conf( void )
 	return TRUE;
 }
 
+/***************************************************************
+* եΥСѴ
+***************************************************************/
+
+void	ver_up_game_conf_1_0_15( void )
+{
+	ver_up_draw_ver_1_2_22();
+}
+
+/***************************************************************
+* եΥСѴ
+***************************************************************/
+
+void	ver_up_game_data_cnf_1_2_22( void )
+{
+	ver_up_draw_ver_1_2_22();
+}
+
 /***************************************************************
 * եΥإåΥ
 * FILE *fp : ե
@@ -263,6 +286,29 @@ bool_t	load_game_conf_parse_ver_1( FILE *fp, const ver_t *ver )
 				u[0].str, u[1].c, u[2].str ) ){
 			flg_err = TRUE;
 		}
+	} else if( (t[0] == RC_TOKEN_ID)
+			&& (t[1] == RC_TOKEN_NUM)
+			&& (t[2] == RC_TOKEN_STR)
+			&& (t[3] == '\n') ){
+		/* foo n str */
+
+		if( !load_game_conf_id_num_str(
+				fp, ver, flg_err_exit,
+				u[0].str, u[1].n, u[2].str ) ){
+			flg_err = TRUE;
+		}
+	} else if( (t[0] == RC_TOKEN_ID)
+			&& (t[1] == RC_TOKEN_STR)
+			&& (t[2] == RC_TOKEN_NUM)
+			&& (t[3] == RC_TOKEN_STR)
+			&& (t[4] == '\n') ){
+		/* foo str n str */
+
+		if( !load_game_conf_id_str_num_str(
+				fp, ver, flg_err_exit,
+				u[0].str, u[1].str, u[2].n, u[3].str ) ){
+			flg_err = TRUE;
+		}
 	} else if( (t[0] == RC_TOKEN_ID)
 			&& (t[1] == RC_TOKEN_CHAR)
 			&& (t[2] == '\n') ){
@@ -321,7 +367,19 @@ bool_t	load_game_conf_set_val(
 		return load_game_conf_screen_size_auto( id, val );
 
 	} else if( strcmp( id, "screen_size_lines" ) == 0 ){
-		return load_game_conf_screen_size_lines( id, val );
+		return load_game_conf_screen_size_row( id, val );
+
+	} else if( strcmp( id, "screen_size_column" ) == 0 ){
+		return load_game_conf_screen_size_col( id, val );
+
+	} else if( strcmp( id, "screen_size_row" ) == 0 ){
+		return load_game_conf_screen_size_row( id, val );
+
+	} else if( strcmp( id, "screen_size_message" ) == 0 ){
+		return load_game_conf_screen_size_message( id, val );
+
+	} else if( strcmp( id, "screen_resize_map" ) == 0 ){
+		return load_game_conf_screen_resize_map( id, val );
 
 	} else if( strcmp( id, "clear_key_buffer_every" ) == 0 ){
 		return load_game_conf_clr_key_buf_every( id, val );
@@ -338,6 +396,9 @@ bool_t	load_game_conf_set_val(
 	} else if( strcmp( id, "multiple_message_line" ) == 0 ){
 		return load_game_conf_mul_line( id, val );
 
+	} else if( strcmp( id, "move_cursor_to_default" ) == 0 ){
+		return load_game_conf_move_crsr_dflt( id, val );
+
 	} else if( strcmp( id, "joystick_number" ) == 0 ){
 		return load_game_conf_joystick_number( id, val );
 
@@ -476,6 +537,79 @@ bool_t	load_game_conf_id_char_str(
 	return FALSE;
 }
 
+/***************************************************************
+* եΥѡ(̻  ʸ)
+* FILE *fp : ե
+* const ver_t *ver : եΥС
+* bool_t flg_err_exit : 顼ǽλ뤫?
+* char *id : ̻
+* long n : 
+* char *str : ʸ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	load_game_conf_id_num_str(
+	FILE *fp, const ver_t *ver, bool_t flg_err_exit,
+	char *id, long n, char *str
+)
+{
+	if( strcmp( id, "set_macro_func" ) == 0 ){
+		return load_game_conf_set_macro_func( id, NULL, n, str );
+
+	} else if( strcmp( id, "set_macro_mouse_click" ) == 0 ){
+		return load_game_conf_set_macro_func( id, NULL, n, str );
+
+	} else if( strcmp( id, "set_macro_mouse_double_click" ) == 0 ){
+		return load_game_conf_set_macro_func( id, NULL, n, str );
+
+	} else if( strcmp( id, "set_macro_mouse_press_and_click" ) == 0 ){
+		return load_game_conf_set_macro_func( id, NULL, n, str );
+
+	} else if( strcmp( id, "set_macro_mouse_press" ) == 0 ){
+		return load_game_conf_set_macro_func( id, NULL, n, str );
+
+	} else if( strcmp( id, "set_macro_mouse_release" ) == 0 ){
+		return load_game_conf_set_macro_func( id, NULL, n, str );
+
+	} else if( strcmp( id, "set_macro_mouse_motion" ) == 0 ){
+		return load_game_conf_set_macro_func( id, NULL, n, str );
+
+	} else {
+		print_lex_err( MSG_S, MSG_ERR_RC_PARSE );
+		return FALSE;
+	}
+
+	return FALSE;
+}
+
+/***************************************************************
+* եΥѡ(̻ ʸ  ʸ)
+* FILE *fp : ե
+* const ver_t *ver : եΥС
+* bool_t flg_err_exit : 顼ǽλ뤫?
+* char *id : ̻
+* char *str1 : ʸ
+* long n : 
+* char *str2 : ʸ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	load_game_conf_id_str_num_str(
+	FILE *fp, const ver_t *ver, bool_t flg_err_exit,
+	char *id, char *str1, long n, char *str2
+)
+{
+	if( strcmp( id, "set_macro_joystick" ) == 0 ){
+		return load_game_conf_set_macro_func( id, str1, n, str2 );
+
+	} else {
+		print_lex_err( MSG_S, MSG_ERR_RC_PARSE );
+		return FALSE;
+	}
+
+	return FALSE;
+}
+
 /***************************************************************
 * եΥѡ(̻ ʸ)
 * FILE *fp : ե
@@ -585,9 +719,51 @@ bool_t	load_game_conf_screen_size_auto( char *id, long val )
 * return : 顼̵ä?
 ***************************************************************/
 
-bool_t	load_game_conf_screen_size_lines( char *id, long val )
+bool_t	load_game_conf_screen_size_col( char *id, long val )
 {
-	set_screen_size_lines( val );
+	set_screen_size_col( val );
+
+	return TRUE;
+}
+
+/***************************************************************
+* եβ̥(Կ)
+* char *id : ̻
+* long val : Կ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	load_game_conf_screen_size_row( char *id, long val )
+{
+	set_screen_size_row( val );
+
+	return TRUE;
+}
+
+/***************************************************************
+* եβ̥(Կ)
+* char *id : ̻
+* long val : Կ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	load_game_conf_screen_size_message( char *id, long val )
+{
+	set_screen_size_message( val );
+
+	return TRUE;
+}
+
+/***************************************************************
+* եβ̥(Կ)
+* char *id : ̻
+* long val : Կ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	load_game_conf_screen_resize_map( char *id, long val )
+{
+	set_flg_resize_map( val );
 
 	return TRUE;
 }
@@ -701,6 +877,20 @@ bool_t	load_game_conf_mul_line( char *id, long val )
 	return TRUE;
 }
 
+/***************************************************************
+* ˥塼Υǥեȹܤإưư뤫
+* char *id : ̻
+* long val : 祤ƥåֹ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	load_game_conf_move_crsr_dflt( char *id, long val )
+{
+	set_flg_move_crsr_dflt( (bool_t)val );
+
+	return TRUE;
+}
+
 /***************************************************************
 * 祤ƥåֹ
 * char *id : ̻
@@ -842,6 +1032,11 @@ bool_t	load_game_conf_vfx_mnstr( flg_vfx_mnstr_t flg, long val )
 
 bool_t	load_game_conf_vfx_wait( char *id, long val )
 {
+#if	1
+	set_vfx_wait( (rate_t)val );
+
+	return TRUE;
+#else
 	rate_t	ret;
 
 	ret = set_vfx_wait( (rate_t)val );
@@ -852,6 +1047,7 @@ bool_t	load_game_conf_vfx_wait( char *id, long val )
 		return TRUE;
 	else
 		return FALSE;
+#endif
 }
 
 /***************************************************************
@@ -1101,6 +1297,66 @@ bool_t	load_game_conf_unset_key( char *id, long c )
 	return TRUE;
 }
 
+/***************************************************************
+* եΥƤ
+* char *id : ̻
+* const char *joy : 祤ƥåμ̻
+* long n : ǥå
+* const char *macro : ޥ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	load_game_conf_set_macro_func(
+	char *id, char *joy, long n, char *macro
+)
+{
+	char	*m;
+
+	if( macro == NULL )
+		return FALSE;
+
+	m = NULL;
+	if( strcmp( id, "set_macro_func" ) == 0 ){
+		m = get_macro_func( n );
+	} else if( strcmp( id, "set_macro_mouse_click" ) == 0 ){
+		m = get_macro_mouse_click( n );
+	} else if( strcmp( id, "set_macro_mouse_double_click" ) == 0 ){
+		m = get_macro_mouse_d_click( n );
+	} else if( strcmp( id, "set_macro_mouse_press_and_click" ) == 0 ){
+		m = get_macro_mouse_p_click( n );
+	} else if( strcmp( id, "set_macro_mouse_press" ) == 0 ){
+		m = get_macro_mouse_press( n );
+	} else if( strcmp( id, "set_macro_mouse_release" ) == 0 ){
+		m = get_macro_mouse_release( n );
+	} else if( strcmp( id, "set_macro_mouse_motion" ) == 0 ){
+		m = get_macro_mouse_motion( n );
+	} else if( strcmp( id, "set_macro_joystick" ) == 0 ){
+		long	j;
+
+		if( joy == NULL )
+			return FALSE;
+
+		for( j = 0; j < JOY_KIND_MAX_N; j++ ){
+			if( strcmp( joy, cv_joystick_kind_to_name( j ) )
+					== 0 ){
+				m = get_macro_joystick( j, n );
+				break;
+			}
+		}
+		if( j >= JOY_KIND_MAX_N )
+			return TRUE;
+	} else {
+		return FALSE;
+	}
+
+	if( m == NULL )
+		return FALSE;
+
+	str_nz_cpy( m, macro, MACRO_MAX_LEN );
+
+	return TRUE;
+}
+
 /***************************************************************
 * եΥ桼˥塼
 * FILE *fp : ե
@@ -1266,6 +1522,9 @@ bool_t	load_game_data_cnf( long n )
 	if( chk_ver_old( &ver, 1, 2, 11 ) )
 		ver_up_game_data_1_2_12();
 
+	if( chk_ver_old( &ver, 1, 2, 21 ) )
+		ver_up_game_data_cnf_1_2_22();
+
 	/* եΥ */
 
 	if( fclose( fp ) == EOF )
@@ -1419,6 +1678,29 @@ bool_t	load_game_data_cnf_parse_ver_1(
 				u[0].str, u[1].c, u[2].str ) ){
 			flg_err = TRUE;
 		}
+	} else if( (t[0] == RC_TOKEN_ID)
+			&& (t[1] == RC_TOKEN_NUM)
+			&& (t[2] == RC_TOKEN_STR)
+			&& (t[3] == '\n') ){
+		/* foo n str */
+
+		if( !load_game_conf_id_num_str(
+				fp, ver, flg_err_exit,
+				u[0].str, u[1].n, u[2].str ) ){
+			flg_err = TRUE;
+		}
+	} else if( (t[0] == RC_TOKEN_ID)
+			&& (t[1] == RC_TOKEN_STR)
+			&& (t[2] == RC_TOKEN_NUM)
+			&& (t[3] == RC_TOKEN_STR)
+			&& (t[4] == '\n') ){
+		/* foo str n str */
+
+		if( !load_game_conf_id_str_num_str(
+				fp, ver, flg_err_exit,
+				u[0].str, u[1].str, u[2].n, u[3].str ) ){
+			flg_err = TRUE;
+		}
 	} else if( (t[0] == RC_TOKEN_ID)
 			&& (t[1] == RC_TOKEN_CHAR)
 			&& (t[2] == '\n') ){
@@ -1652,6 +1934,7 @@ bool_t	save_game_conf_contents( FILE *fp, const ver_t *ver )
 	flg_err |= !save_game_conf_vfx( fp, ver );
 	flg_err |= !save_game_conf_color( fp, ver );
 	flg_err |= !save_game_conf_key_tab( fp, ver );
+	flg_err |= !save_game_conf_func( fp, ver );
 	flg_err |= !save_game_conf_user_menu( fp, ver );
 
 	if( flg_err )
@@ -1726,8 +2009,14 @@ bool_t	save_game_conf_screen_size( FILE *fp, const ver_t *ver )
 
 	fprintf( fp, "screen_size_auto = %ld\n",
 			(long)chk_flg_screen_size_auto() );
-	fprintf( fp, "screen_size_lines = %ld\n",
-			(long)get_screen_size_lines() );
+	fprintf( fp, "screen_size_column = %ld\n",
+			(long)get_screen_size_col() );
+	fprintf( fp, "screen_size_row = %ld\n",
+			(long)get_screen_size_row() );
+	fprintf( fp, "screen_size_message = %ld\n",
+			(long)get_screen_size_message() );
+	fprintf( fp, "screen_resize_map = %ld\n",
+			(long)chk_flg_resize_map() );
 
 	return TRUE;
 }
@@ -1808,6 +2097,11 @@ bool_t	save_game_conf_flg( FILE *fp, const ver_t *ver )
 	fprintf( fp, "multiple_message_line = %ld\n",
 			(long)chk_msg_mul_line() );
 
+	fprintf( fp, "\n" );
+	fprintf( fp, "# move cursor to default\n" );
+	fprintf( fp, "move_cursor_to_default = %ld\n",
+			(long)get_flg_move_crsr_dflt() );
+
 	return TRUE;
 }
 
@@ -2171,20 +2465,185 @@ bool_t	save_game_conf_key_tab( FILE *fp, const ver_t *ver )
 	for( i = 0; i < KEY_TAB_MAX_N; i++ ){
 		char	key[15 + 1];
 
-		if( key_tab[i] != NULL ){
-			if( isprint( (char)i ) ){
-				sn_printf( key, 15, "%c", (char)i );
-			} else if( ('A' - '@' <= i) && (i <= 'Z' - '@') ){
-				sn_printf( key, 15, "^%c", (char)(i + '@') );
-			} else {
-				sn_printf( key, 15, "\\%04o", (char)i );
-			}
+		if( key_tab[i] == NULL )
+			continue;
 
-			fprintf( fp, "set_key '%s'",
-					cv_esc_quote_str( key ) );
-			fprintf( fp, " \"%s\"\n",
-					cv_esc_quote_str( key_tab[i] ) );
+		if( isprint( (char)i ) ){
+			sn_printf( key, 15, "%c", (char)i );
+		} else if( ('A' - '@' <= i) && (i <= 'Z' - '@') ){
+			sn_printf( key, 15, "^%c", (char)(i + '@') );
+		} else {
+			sn_printf( key, 15, "\\%04o", (char)i );
 		}
+
+		fprintf( fp, "set_key '%s'",
+				cv_esc_quote_str( key ) );
+		fprintf( fp, " \"%s\"\n",
+				cv_esc_quote_str( key_tab[i] ) );
+	}
+
+	return TRUE;
+}
+
+/***************************************************************
+* եΥƤΥ
+* FILE *fp : ե
+* const ver_t *ver : եΥС
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	save_game_conf_func( FILE *fp, const ver_t *ver )
+{
+	long	i, j;
+	const char	*m;
+
+	if( fp == NULL )
+		return FALSE;
+	if( ver == NULL )
+		return FALSE;
+
+	fprintf( fp, "\n" );
+	fprintf( fp, "# bind macro to function key\n" );
+	fprintf( fp, "# ex.\n" );
+	fprintf( fp, "# set_macro_func no. \"command\"\n" );
+	fprintf( fp, "\n" );
+
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		m = get_macro_func( i );
+		if( m == NULL )
+			break;
+
+		if( !save_game_conf_func_sub( fp, ver,
+				"set_macro_func",
+				NULL, i, m ) ){
+			return FALSE;
+		}
+	}
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		m = get_macro_mouse_click( i );
+		if( m == NULL )
+			break;
+
+		if( !save_game_conf_func_sub( fp, ver,
+				"set_macro_mouse_click",
+				NULL, i, m ) ){
+			return FALSE;
+		}
+	}
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		m = get_macro_mouse_d_click( i );
+		if( m == NULL )
+			break;
+
+		if( !save_game_conf_func_sub( fp, ver,
+				"set_macro_mouse_double_click",
+				NULL, i, m ) ){
+			return FALSE;
+		}
+	}
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		m = get_macro_mouse_p_click( i );
+		if( m == NULL )
+			break;
+
+		if( !save_game_conf_func_sub( fp, ver,
+				"set_macro_mouse_press_and_click",
+				NULL, i, m ) ){
+			return FALSE;
+		}
+	}
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		m = get_macro_mouse_press( i );
+		if( m == NULL )
+			break;
+
+		if( !save_game_conf_func_sub( fp, ver,
+				"set_macro_mouse_press",
+				NULL, i, m ) ){
+			return FALSE;
+		}
+	}
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		m = get_macro_mouse_release( i );
+		if( m == NULL )
+			break;
+
+		if( !save_game_conf_func_sub( fp, ver,
+				"set_macro_mouse_release",
+				NULL, i, m ) ){
+			return FALSE;
+		}
+	}
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
+		m = get_macro_mouse_motion( i );
+		if( m == NULL )
+			break;
+
+		if( !save_game_conf_func_sub( fp, ver,
+				"set_macro_mouse_motion",
+				NULL, i, m ) ){
+			return FALSE;
+		}
+	}
+	for( j = 0; j < LOOP_MAX_100; j++ ){
+		for( i = 0; i < LOOP_MAX_1000; i++ ){
+			const char	*joy_name;
+
+			m = get_macro_joystick( j, i );
+			if( m == NULL )
+				break;
+
+			joy_name = cv_joystick_kind_to_name( j );
+			if( joy_name == NULL )
+				continue;
+
+			if( !save_game_conf_func_sub( fp, ver,
+					"set_macro_joystick",
+					joy_name, i, m ) ){
+				return FALSE;
+			}
+		}
+	}
+
+	return TRUE;
+}
+
+/***************************************************************
+* եΥƤΥ
+* FILE *fp : ե
+* const ver_t *ver : եΥС
+* const char *tag : ̻
+* const char *joy : 祤ƥåμ̻
+* long n : ǥå
+* const char *macro : ޥ
+* return : 顼̵ä?
+***************************************************************/
+
+bool_t	save_game_conf_func_sub(
+	FILE *fp,
+	const ver_t *ver,
+	const char *tag,
+	const char *joy,
+	long n,
+	const char *macro
+)
+{
+	if( fp == NULL )
+		return FALSE;
+	if( ver == NULL )
+		return FALSE;
+	if( macro == NULL )
+		return FALSE;
+
+	if( macro[0] == '\0' )
+		return TRUE;
+
+	if( joy == NULL ){
+		fprintf( fp, "%s %ld \"%s\"\n",
+				tag, n, cv_esc_quote_str( macro ) );
+	} else {
+		fprintf( fp, "%s \"%s\" %ld \"%s\"\n",
+				tag, joy, n, cv_esc_quote_str( macro ) );
 	}
 
 	return TRUE;
@@ -2443,3 +2902,11 @@ int	cv_name_to_color( char *name )
 
 	return COLOR_WHITE;
 }
+
+/***************************************************************
+* ꡼å
+***************************************************************/
+
+void	check_memory_gf_conf_c( void )
+{
+}
