@@ -35,7 +35,7 @@ using namespace System;
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: lnl-dos.cpp,v 1.4 2006/01/12 08:51:40 bowkenken Exp $
+* $Id: lnl-dos.cpp,v 1.5 2007/03/02 05:19:48 bowkenken Exp $
 *****************************************************************/
 
 #include	"../gmain.h"
@@ -44,19 +44,10 @@ using namespace System;
 
 /**/
 
-int main( void )
-{
-	main_dos();
-
-	return 0;
-}
-
-/**/
-
-int _tmain()
+int _tmain(int argc, _TCHAR* argv[] )
 {
     // TODO: Υץ ɤȼΥɤ֤Ƥ
-	main();
+	main_dos( argc, (char **)argv );
 
 	return 0;
 }
