@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2009 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: main-dos.c,v 1.16 2008/04/03 14:02:52 bowkenken Exp $
+* $Id: main-dos.c,v 1.19 2009/06/26 16:58:16 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -53,6 +53,14 @@ int	main_dos( int argc, char **argv )
 	return EXIT_FAILURE;
 }
 
+/***************************************************************
+* ƥåѿꥻå
+***************************************************************/
+
+void	reset_static_main_dos( void )
+{
+}
+
 /***************************************************************
 * λ
 * long code : λ
@@ -62,3 +70,11 @@ void	gexit( long code )
 {
 	exit( code );
 }
+
+/***************************************************************
+* ꡼å
+***************************************************************/
+
+void	check_memory_main_dos_c( void )
+{
+}
