@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2009 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: mmenu-tab.h,v 1.38 2008/10/05 10:59:55 bowkenken Exp $
+* $Id: mmenu-tab.h,v 1.50 2009/08/16 05:15:38 bowkenken Exp $
 ***************************************************************/
 
 #ifndef	MMENU_TAB_H
@@ -96,9 +96,9 @@ menu_t	ls_menu_main_race[] = {
 		FLG_NULL,
 	},
 	{
-		"/main race/bibit",
+		"/main race/vivid",
 		"<Item>",
-		'B',
+		'V',
 		"",
 		N_MSG_MAIN_RACE_VIVID, "",
 		cb_menu_main_race, (void *)MAIN_RACE_VIVID,
@@ -432,6 +432,15 @@ menu_t	ls_menu_main[] = {
 		cb_menu_ls_msg, NULL,
 		FLG_NULL,
 	},
+	{
+		"/misc/info/list key item",
+		"<Item>",
+		'K',
+		"",
+		N_MSG_MENU_LS_KEY_ITEM, "",
+		cb_menu_ls_key_item, NULL,
+		FLG_NULL,
+	},
 	{
 		"/misc/info/list monster",
 		"<Item>",
@@ -786,12 +795,39 @@ menu_t	ls_menu_main[] = {
 		FLG_NULL,
 	},
 	{
-		"/misc/config/screen/window/size/lines",
+		"/misc/config/screen/window/size/column",
 		"<OpenWinItem>",
-		'L',
+		'C',
 		"",
-		N_MSG_MENU_SCREEN_SIZE_LINES, "",
-		cb_menu_screen_size_lines, NULL,
+		N_MSG_MENU_SCREEN_SIZE_COL, "",
+		cb_menu_screen_size_col, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/screen/window/size/row",
+		"<OpenWinItem>",
+		'R',
+		"",
+		N_MSG_MENU_SCREEN_SIZE_ROW, "",
+		cb_menu_screen_size_row, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/screen/window/size/message",
+		"<OpenWinItem>",
+		'S',
+		"",
+		N_MSG_MENU_SCREEN_SIZE_MESSAGE, "",
+		cb_menu_screen_size_message, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/screen/window/size/map",
+		"<CheckItem>",
+		'M',
+		"",
+		N_MSG_MENU_SCREEN_SIZE_MAP, "",
+		cb_menu_screen_size_map, NULL,
 		FLG_NULL,
 	},
 /* VFX begin */
@@ -1046,7 +1082,34 @@ menu_t	ls_menu_main[] = {
 		'G',
 		"",
 		N_MSG_MENU_GUI, "",
-		NULL, NULL,
+		cb_menu_gui, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/gui/move cursor to default",
+		"<CheckItem>",
+		'C',
+		"",
+		N_MSG_MENU_MOVE_CRSR_DFLT, "",
+		cb_menu_move_crsr_dflt, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/gui/replay music",
+		"<Item>",
+		'R',
+		"p",
+		N_MSG_MENU_REPLAY_MUSIC, "",
+		cb_menu_replay_music, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/config/gui/mouse",
+		"<Branch>",
+		'M',
+		"",
+		N_MSG_MENU_MOUSE, "",
+		cb_menu_mouse, NULL,
 		FLG_NULL,
 	},
 	{
@@ -1103,6 +1166,15 @@ menu_t	ls_menu_main[] = {
 		cb_menu_joystick_sel_kind, (void *)JOY_KIND_RAP3,
 		FLG_NULL,
 	},
+	{
+		"/misc/config/gui/joystick/macro",
+		"<Branch>",
+		'M',
+		"",
+		N_MSG_MENU_JOYSTICK_MACRO, "",
+		cb_menu_joystick_macro_init, NULL,
+		FLG_NULL,
+	},
 	{
 		"/misc/config/user menu select",
 		"<Branch>",
@@ -1198,6 +1270,24 @@ menu_t	ls_menu_main[] = {
 		NULL, NULL,
 		FLG_NULL,
 	},
+	{
+		"/misc/system/wizard/teleport party",
+		"<Item>",
+		'P',
+		"",
+		N_MSG_MENU_WIZARD_TELEPORT_PARTY, "",
+		cb_menu_wizard_teleport_party, NULL,
+		FLG_NULL,
+	},
+	{
+		"/misc/system/wizard/recall",
+		"<OpenWinItem>",
+		'R',
+		"",
+		N_MSG_MENU_WIZARD_RECALL, "",
+		cb_menu_wizard_recall, NULL,
+		FLG_NULL,
+	},
 	{
 		"/misc/system/wizard/on light",
 		"<Item>",
@@ -1342,6 +1432,15 @@ menu_t	ls_menu_main[] = {
 		cb_menu_wizard_last_scene, (void *)LAST_SCENE_N_XEX,
 		FLG_NULL,
 	},
+	{
+		"/misc/system/wizard/scene/last boss/xx",
+		"<Item>",
+		'F',
+		"",
+		N_MSG_MENU_WIZARD_LAST_SCENE_XX, "",
+		cb_menu_wizard_last_scene, (void *)LAST_SCENE_N_XX_WORDS,
+		FLG_NULL,
+	},
 	{
 		"/misc/system/wizard/scene/game over",
 		"<Item>",
@@ -2019,6 +2118,15 @@ menu_t	ls_menu_item_sel[] = {
 		cb_menu_item_drop, (void *)ACT_KIND_ITEM_DROP,
 		FLG_NULL,
 	},
+	{
+		"/item sel/alias",
+		"<OpenWinItem>",
+		'N',
+		"",
+		N_MSG_MENU_ITEM_ALIAS, "",
+		cb_menu_item_alias, (void *)ACT_KIND_NULL,
+		FLG_NULL,
+	},
 	{
 		"/item sel/check",
 		"<Item>",
