@@ -34,6 +34,8 @@ void	set_act_mnstr_succubus( mnstr_t *p );
 void	set_act_mnstr_exel( mnstr_t *p );
 void	set_act_mnstr_exeler( mnstr_t *p );
 void	set_act_mnstr_xex( mnstr_t *p );
+void	set_act_mnstr_xx( mnstr_t *p );
+void	check_memory_mnstr_act_c( void );
 
 #ifdef __cplusplus
 }
