@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: mnstr-act.c,v 1.30 2009/06/26 16:58:16 bowkenken Exp $
+* $Id: mnstr-act.c,v 1.31 2009/07/23 11:58:15 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -443,14 +443,6 @@ void	set_act_mnstr_phoenix( mnstr_t *p )
 			fx_resurrection( pty->mbr[i], TRUE );
 	}
 
-	/* ꥿ */
-
-	p->work.dismissal_turn--;
-	if( p->work.dismissal_turn <= 0 ){
-		fx_dismissal( p );
-		return;
-	}
-
 	/* Ĥ̤Υ */
 
 	set_act_mnstr_std( p );
@@ -487,14 +479,6 @@ void	set_act_mnstr_unicorn( mnstr_t *p )
 		fx_cure_all_stat( pty->mbr[i] );
 	}
 
-	/* ꥿ */
-
-	p->work.dismissal_turn--;
-	if( p->work.dismissal_turn <= 0 ){
-		fx_dismissal( p );
-		return;
-	}
-
 	/* Ĥ̤Υ */
 
 	set_act_mnstr_std( p );
