@@ -5,11 +5,13 @@ extern "C" {
 #endif
 
 void	init_spell( void );
+void	reset_static_spell( void );
 void	init_spell_debug( void );
 void	init_spell_new_mbr( long ls_mbr_n );
 void	init_spell_dflt( void );
 void	init_spell_nin( void );
 char	*cb_menu_sel_spell( menu_t **pp, menu_t **dflt );
+char	*menu_sel_spell( mbr_t *mbr );
 char	*cb_menu_dflt_spell_mag( menu_t **pp, menu_t **dflt );
 char	*cb_menu_dflt_spell_sor( menu_t **pp, menu_t **dflt );
 char	*cb_menu_dflt_spell_enc( menu_t **pp, menu_t **dflt );
@@ -24,6 +26,7 @@ char	*cb_menu_spell( menu_t **pp, menu_t **dflt );
 char	*cb_menu_spell_extent( menu_t **pp, menu_t **dflt );
 char	*menu_spell_trgt( mbr_t *mbr );
 void	chr_cast_spell( chr_t *chr, spell_tab_t *tab, const char *fmt );
+void	play_sound_spell( chr_t *chr, spell_kind_t kind );
 void	chr_cast_spell_normal(
 	chr_t *chr, bool_t flg_item,
 	spell_tab_t *tab, rate_t rate
@@ -77,13 +80,15 @@ void	counter_magic(
 );
 bool_t	chk_spell( chr_t *o_p, spell_tab_t *tab, bool_t flg_item );
 bool_t	chk_cast_spell_trgt( spell_kind_t spell, trgt_kind_t trgt );
+fx_kind_t	cv_spell_to_fx( spell_kind_t spell );
+flg_stat_t	cv_spell_to_stat( spell_kind_t spell );
 bool_t	chk_bar_limit( chr_t *chr );
 bool_t	chk_bar_limit_lev( chr_t *chr );
 bool_t	chk_can_chr_cast_stat( chr_t *chr );
 bool_t	chk_can_chr_cast_lev( chr_t *chr, spell_kind_t kind );
 bool_t	chk_chr_learned_spell( chr_t *chr, spell_kind_t kind );
 bool_t	set_chr_learned_spell( chr_t *chr, spell_kind_t kind, bool_t flg );
-void	learn_spell_pri( chr_t *chr );
+bool_t	learn_spell_pri( chr_t *chr );
 void	learn_spell_sha( chr_t *chr );
 void	learn_spell_bar( chr_t *chr );
 bool_t	chk_chr_learned_spell_nin(
@@ -91,6 +96,7 @@ bool_t	chk_chr_learned_spell_nin(
 	bool_t flg_test
 );
 bool_t	chk_learn_spell_item( chr_t *chr, spell_kind_t kind );
+void	arrange_cast_tab_1( long trgt_ls_mbr_n, long d );
 void	arrange_cast_tab( long trgt_ls_mbr_n, long d );
 void	remove_cast_tab( long trgt_ls_mbr_n );
 bool_t	copy_cast_tab( cast_tab_t dst[], cast_tab_t src[] );
@@ -99,6 +105,7 @@ cast_tab_t	*get_cast_tab( chr_t *chr, spell_kind_t kind );
 char	*get_spell_name( spell_kind_t kind );
 spell_tab_t	*get_spell_tab( spell_kind_t kind );
 gold_t	get_price_spell( spell_kind_t kind );
+void	check_memory_spell_c( void );
 
 #ifdef __cplusplus
 }
