@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: spell.c,v 1.195 2009/06/23 03:23:26 bowkenken Exp $
+* $Id: spell.c,v 1.205 2009/08/08 21:02:29 bowkenken Exp $
 ***************************************************************/
 
 #define	SPELL_C
@@ -49,33 +49,41 @@
 
 #define	SKELETON_WARRIOR_STD_N	2
 
-#define	DISMISSAL_TURN_BONE	64
-#define	DISMISSAL_TURN_SKELETON	96
-#define	DISMISSAL_TURN_UNICORN	64
-#define	DISMISSAL_TURN_EFREET	64
-#define	DISMISSAL_TURN_DJINNI	64
-#define	DISMISSAL_TURN_PHOENIX	32
-#define	DISMISSAL_TURN_ELEMENTAL	32
+#define	DISMISSAL_TURN_BONE	128
+#define	DISMISSAL_TURN_SKELETON	128
+#define	DISMISSAL_TURN_UNICORN	128
+#define	DISMISSAL_TURN_EFREET	128
+#define	DISMISSAL_TURN_DJINNI	128
+#define	DISMISSAL_TURN_PHOENIX	128
+#define	DISMISSAL_TURN_ELEMENTAL	64
 
 #define	SPELL_NIN_RATE	((rate_t)80)
-#define	SPELL_SHA_DIV_RESI_RATE	4
+#define	SPELL_SHA_RESI_DIV_RATE	((rate_t)400)
 
 #define	BAR_LIMIT_LEV	10
 
 /**/
 
 static cast_tab_t	cast_tab[LS_MBR_MAX_N][SPELL_KIND_MAX_N];
+check_memory_def( check_memory_spell_c_cast_tab )
 
 static mbr_t	*g_sel_mbr;
 static spell_tab_t	*g_sel_spell;
+check_memory_def( check_memory_spell_c_g_sel_mbr )
 
 static menu_t	*g_dflt_spell_abl[LS_MBR_MAX_N];
+check_memory_def( check_memory_spell_c_g_dflt_spell_abl )
 static menu_t	*g_dflt_spell[LS_MBR_MAX_N][ABL_KIND_MAX_N];
+check_memory_def( check_memory_spell_c_g_dflt_spell )
 static menu_t	*g_dflt_spell_extent[LS_MBR_MAX_N][ABL_KIND_MAX_N];
+check_memory_def( check_memory_spell_c_g_dflt_spell_extent )
 static abl_kind_t	g_sel_spell_abl;
+check_memory_def( check_memory_spell_c_g_sel_spell_abl )
 
 static long	g_vfx_n;
+check_memory_def( check_memory_spell_c_g_vfx_n )
 static pos_t	g_vfx_bgn[VFX_ARW_MAX_N], g_vfx_end[VFX_ARW_MAX_N];
+check_memory_def( check_memory_spell_c_g_vfx_bgn )
 
 /**/
 
@@ -277,38 +285,68 @@ char	*cb_menu_sel_spell( menu_t **pp, menu_t **dflt )
 {
 	mbr_t	*pre_sel_mbr;
 	pos_t	*crsr;
+	bool_t	flg_sel_mbr;
+	char	*str_goto;
+
+	pre_sel_mbr = static_cb_menu_sel_spell__pre_sel_mbr;
+
+	set_menu_dflt_main( *pp );
+
+	crsr = get_crsr();
+	g_sel_mbr = get_mbr( crsr->x, crsr->y );
+	if( g_sel_mbr == NULL )
+		flg_sel_mbr = TRUE;
+	else
+		flg_sel_mbr = FALSE;
+
+	do {
+		if( flg_sel_mbr ){
+			char	*str;
+
+			g_sel_mbr = pre_sel_mbr;
+
+			str = exec_menu_sel_mbr( &g_sel_mbr, NULL,
+					FALSE, FALSE );
+			if( str != NULL )
+				return STR_MENU_ERR;
+		}
+		if( g_sel_mbr == NULL )
+			return STR_MENU_ERR;
+
+		pre_sel_mbr = g_sel_mbr;
+		static_cb_menu_sel_spell__pre_sel_mbr = pre_sel_mbr;
+
+		str_goto = menu_sel_spell( g_sel_mbr );
+		if( str_goto == NULL )
+			return NULL;
+		if( strcmp( str_goto, STR_MENU_CANCEL ) == 0 )
+			continue;
+		if( strcmp( str_goto, STR_MENU_ERR ) == 0 )
+			return str_goto;
+	} while( flg_sel_mbr );
+
+	return NULL;
+}
+
+/**/
+
+char	*menu_sel_spell( mbr_t *mbr )
+{
+	menu_t *dflt;
 	menu_t	*ls;
 	unsigned char	key;
 	bool_t	flg;
 	bool_t	flg_learn;
 	long	i;
 
-	pre_sel_mbr = static_cb_menu_sel_spell__pre_sel_mbr;
-
-	crsr = get_crsr();
-	g_sel_mbr = get_mbr( crsr->x, crsr->y );
-	if( g_sel_mbr == NULL ){
-		char	*str;
-
-		g_sel_mbr = pre_sel_mbr;
-
-		str = exec_menu_sel_mbr( &g_sel_mbr, NULL, FALSE, FALSE );
-		if( str != NULL )
-			return STR_MENU_ERR;
-	}
-	if( g_sel_mbr == NULL )
+	if( mbr == NULL )
 		return STR_MENU_ERR;
 
-	pre_sel_mbr = g_sel_mbr;
-	static_cb_menu_sel_spell__pre_sel_mbr = pre_sel_mbr;
-
-	if( !chk_can_chr_cast_stat( g_sel_mbr ) ){
-		print_msg( FLG_NULL, MSG_ERR_SPELL, g_sel_mbr->name );
+	if( !chk_can_chr_cast_stat( mbr ) ){
+		print_msg( FLG_NULL, MSG_ERR_SPELL, mbr->name );
 		return STR_MENU_CANCEL;
 	}
 
-	set_menu_dflt_main( *pp );
-
 	ls = ls_menu_spell;
 	clr_flg_menu( ls, NULL, FLG_MENU_ALL );
 
@@ -339,7 +377,7 @@ char	*cb_menu_sel_spell( menu_t **pp, menu_t **dflt )
 		key++;
 
 		kind = (spell_kind_t)(long)(ls[i].arg);
-		if( chk_can_chr_cast_lev( g_sel_mbr, kind ) ){
+		if( chk_can_chr_cast_lev( mbr, kind ) ){
 			ls[i].flg &= ~FLG_MENU_HIDDEN;
 			flg_learn = TRUE;
 		} else {
@@ -349,71 +387,69 @@ char	*cb_menu_sel_spell( menu_t **pp, menu_t **dflt )
 
 	flg = FALSE;
 
-	if( get_lev( g_sel_mbr, ABL_KIND_MAG ) > 0 ){
+	if( get_lev( mbr, ABL_KIND_MAG ) > 0 ){
 		clr_flg_menu( ls, "MAG", FLG_MENU_HIDDEN );
 		flg = TRUE;
 	} else {
 		set_flg_menu( ls, "MAG", FLG_MENU_HIDDEN );
 	}
 
-	if( get_lev( g_sel_mbr, ABL_KIND_SOR ) > 0 ){
+	if( get_lev( mbr, ABL_KIND_SOR ) > 0 ){
 		clr_flg_menu( ls, "SOR", FLG_MENU_HIDDEN );
 		flg = TRUE;
 	} else {
 		set_flg_menu( ls, "SOR", FLG_MENU_HIDDEN );
 	}
 
-	if( get_lev( g_sel_mbr, ABL_KIND_ENC ) > 0 ){
+	if( get_lev( mbr, ABL_KIND_ENC ) > 0 ){
 		clr_flg_menu( ls, "ENC", FLG_MENU_HIDDEN );
 		flg = TRUE;
 	} else {
 		set_flg_menu( ls, "ENC", FLG_MENU_HIDDEN );
 	}
 
-	if( get_lev( g_sel_mbr, ABL_KIND_SUM ) > 0 ){
+	if( get_lev( mbr, ABL_KIND_SUM ) > 0 ){
 		clr_flg_menu( ls, "SUM", FLG_MENU_HIDDEN );
 		flg = TRUE;
 	} else {
 		set_flg_menu( ls, "SUM", FLG_MENU_HIDDEN );
 	}
 
-	if( get_lev( g_sel_mbr, ABL_KIND_PRI ) > 0 ){
+	if( get_lev( mbr, ABL_KIND_PRI ) > 0 ){
 		clr_flg_menu( ls, "PRI", FLG_MENU_HIDDEN );
 		flg = TRUE;
 	} else {
 		set_flg_menu( ls, "PRI", FLG_MENU_HIDDEN );
 	}
 
-	if( get_lev( g_sel_mbr, ABL_KIND_SHA ) > 0 ){
+	if( get_lev( mbr, ABL_KIND_SHA ) > 0 ){
 		clr_flg_menu( ls, "SHA", FLG_MENU_HIDDEN );
 		flg = TRUE;
 	} else {
 		set_flg_menu( ls, "SHA", FLG_MENU_HIDDEN );
 	}
 
-	if( get_lev( g_sel_mbr, ABL_KIND_BAR ) > 0 ){
+	if( get_lev( mbr, ABL_KIND_BAR ) > 0 ){
 		clr_flg_menu( ls, "BAR", FLG_MENU_HIDDEN );
 		flg = TRUE;
 	} else {
 		set_flg_menu( ls, "BAR", FLG_MENU_HIDDEN );
 	}
 
-	if( get_lev( g_sel_mbr, ABL_KIND_NIN ) > 0 ){
+	if( get_lev( mbr, ABL_KIND_NIN ) > 0 ){
 		clr_flg_menu( ls, "NIN", FLG_MENU_HIDDEN );
 		flg = TRUE;
 	} else {
 		set_flg_menu( ls, "NIN", FLG_MENU_HIDDEN );
 	}
 
-	*pp = ls;
-	*dflt = g_dflt_spell_abl[g_sel_mbr->ls_mbr_n];
-
 	if( !flg ){
-		print_msg( FLG_NULL, MSG_ERR_SPELL, g_sel_mbr->name );
+		print_msg( FLG_NULL, MSG_ERR_SPELL, mbr->name );
 		return STR_MENU_CANCEL;
 	}
 
-	return NULL;
+	dflt = g_dflt_spell_abl[mbr->ls_mbr_n];
+	return exec_menu( ls, dflt, NULL );
 }
 
 /**/
@@ -771,25 +807,30 @@ void	chr_cast_spell( chr_t *chr, spell_tab_t *tab, const char *fmt )
 	if( (tab->resi_down > 0) && (resi < 1) )
 		resi = 1;
 
-	{
+	if( resi > 0 ){
 		long	resi_div;
 
 		resi_div = get_lev( chr, ABL_KIND_SHA )
 				+ get_lev( chr, ABL_KIND_CHA );
-		resi_div /= SPELL_SHA_DIV_RESI_RATE;
+		resi_div *= SPELL_SHA_RESI_DIV_RATE;
+		resi_div /= _100_PERCENT;
 		if( resi_div < 1 )
 			resi_div = 1;
 
 		resi *= _100_PERCENT;
 		resi /= resi_div;
+		if( resi < 1 )
+			resi = 1;
 	}
 
 	if( chr->abl.hp.n < hp ){
+		clr_chr_trgt_act( chr, TRUE );
 		print_msg_mbr( chr, FLG_NULL, MSG_ERR_HP_SHORT, chr->name,
 				hp - chr->abl.hp.n );
 		return;
 	}
 	if( chr->abl.mp.n < mp ){
+		clr_chr_trgt_act( chr, TRUE );
 		print_msg_mbr( chr, FLG_NULL, MSG_ERR_MP_SHORT, chr->name,
 				mp - chr->abl.mp.n );
 		return;
@@ -805,20 +846,16 @@ void	chr_cast_spell( chr_t *chr, spell_tab_t *tab, const char *fmt )
 		if( kind >= RESI_KIND_MAX_N )
 			return;
 
-		d = chr->resi[kind].n - ((resi / _100_PERCENT) + 1);
+		d = chr->resi[kind].n - resi;
 		if( d <= 0 ){
+			clr_chr_trgt_act( chr, TRUE );
 			print_msg_mbr( chr, FLG_NULL,
 					MSG_ERR_RESI_SHORT,
-					chr->name, -d + 1 );
+					chr->name, (long)-d + 1 );
 			return;
 		}
 
-		add_resi( chr, kind, -(resi / _100_PERCENT), TRUE );
-		if( (resi % _100_PERCENT) != 0 ){
-			if( rate_randm( resi % _100_PERCENT ) ){
-				add_resi( chr, kind, -1, TRUE );
-			}
-		}
+		add_resi( chr, kind, -resi, TRUE );
 	}
 
 	add_mp( chr, -mp, TRUE );
@@ -1027,6 +1064,7 @@ void	chr_cast_spell_blast(
 	long	nx, ny;
 	long	xx, yy;
 	bool_t	flg_actual;
+	bool_t	flg_pet;
 	chr_kind_t	kind;
 	chr_t	*trgt_chr;
 
@@ -1044,6 +1082,11 @@ void	chr_cast_spell_blast(
 	else
 		kind = CHR_KIND_NULL;
 
+	if( is_pet( (pet_t *)(chr->trgt.p) ) )
+		flg_pet = TRUE;
+	else
+		flg_pet = FALSE;
+
 	if( is_mnstr( chr ) ){
 		if( chk_flg( chr->flg_map, FLG_MAP_CHR_FIND ) ){
 			call_game_sound_play(
@@ -1097,6 +1140,8 @@ void	chr_cast_spell_blast(
 				if( p == chr )
 					continue;
 
+				if( flg_pet )
+					break;
 				if( flg_actual )
 					break;
 				if( is_mnstr( chr ) )
@@ -1656,6 +1701,7 @@ void	chr_cast_spell_sub(
 
 	if( flg_chr ){
 		if( d_p->gold < tab->gold ){
+			clr_chr_trgt_act( chr, TRUE );
 			print_msg_mbr( d_p, FLG_NULL, MSG_ERR_GOLD_SHORT,
 					d_p->name, tab->gold - d_p->gold );
 
@@ -1774,11 +1820,6 @@ void	chr_cast_spell_sub(
 			fx_disarm_trap( get_trap( x, y ), chr );
 		}
 		break;
-	case SPELL_KIND_DISPEL_MAGIC:
-/*@@@*/
-		break;
-	case SPELL_KIND_MIRROR_IMAGE:
-/*@@@*/
 	case SPELL_KIND_ANIMATE_OBJECTS:
 /*@@@*/
 		break;
@@ -1832,6 +1873,12 @@ void	chr_cast_spell_sub(
 	case SPELL_KIND_PASSWALL:
 		fx_std( (chr_t *)p, cv_spell_to_fx( tab->kind ) );
 		break;
+	case SPELL_KIND_DISPEL_MAGIC:
+/*@@@*/
+		break;
+	case SPELL_KIND_MIRROR_IMAGE:
+/*@@@*/
+		break;
 	case SPELL_KIND_POLYMORPH_OTHER:
 		fx_polymorph( (chr_t *)p );
 		break;
@@ -2196,6 +2243,9 @@ void	chr_cast_spell_sub(
 		fx_summon_monster( chr, MNSTR_KIND_SHADE, 1,
 				DISMISSAL_TURN_ELEMENTAL );
 		break;
+	case SPELL_KIND_DISMISSAL_ELEMENTAL:
+		fx_dismissal( (chr_t *)p );
+		break;
 	case SPELL_KIND_SHA_MAX_N:
 		break;
 /* BAR */
@@ -2400,8 +2450,6 @@ bool_t	chk_cast_spell_trgt( spell_kind_t spell, trgt_kind_t trgt )
 	case SPELL_KIND_KNOCK:
 	case SPELL_KIND_LOCK:
 	case SPELL_KIND_DISARM_TRAP:
-	case SPELL_KIND_DISPEL_MAGIC:
-	case SPELL_KIND_MIRROR_IMAGE:
 	case SPELL_KIND_ANIMATE_OBJECTS:
 	case SPELL_KIND_SHELTER:
 	case SPELL_KIND_CREATE_FOOD:
@@ -2441,6 +2489,8 @@ bool_t	chk_cast_spell_trgt( spell_kind_t spell, trgt_kind_t trgt )
 		if( (trgt != TRGT_KIND_MBR) && (trgt != TRGT_KIND_MNSTR) )
 			return FALSE;
 		break;
+	case SPELL_KIND_DISPEL_MAGIC:
+	case SPELL_KIND_MIRROR_IMAGE:
 	case SPELL_KIND_POLYMORPH_OTHER:
 	case SPELL_KIND_POLYMORPH_SELF:
 	case SPELL_KIND_STEALTH:
@@ -2613,6 +2663,10 @@ bool_t	chk_cast_spell_trgt( spell_kind_t spell, trgt_kind_t trgt )
 	case SPELL_KIND_SUMMON_WILL_O_WISP:
 	case SPELL_KIND_SUMMON_SHADE:
 		break;
+	case SPELL_KIND_DISMISSAL_ELEMENTAL:
+		if( trgt != TRGT_KIND_MNSTR )
+			return FALSE;
+		break;
 	case SPELL_KIND_SHA_MAX_N:
 		break;
 /* BAR */
@@ -3310,8 +3364,6 @@ char	*get_spell_vfx_str( spell_kind_t spell_kind )
 		return "+^";
 	case SPELL_KIND_DISARM_TRAP:
 		return "^*";
-	case SPELL_KIND_DISPEL_MAGIC:
-	case SPELL_KIND_MIRROR_IMAGE:
 	case SPELL_KIND_ANIMATE_OBJECTS:
 		break;
 	case SPELL_KIND_SHELTER:
@@ -3332,6 +3384,8 @@ char	*get_spell_vfx_str( spell_kind_t spell_kind )
 	case SPELL_KIND_VANISH:
 	case SPELL_KIND_DARKVISION:
 	case SPELL_KIND_PASSWALL:
+	case SPELL_KIND_DISPEL_MAGIC:
+	case SPELL_KIND_MIRROR_IMAGE:
 	case SPELL_KIND_POLYMORPH_OTHER:
 	case SPELL_KIND_POLYMORPH_SELF:
 	case SPELL_KIND_STEALTH:
@@ -3492,6 +3546,8 @@ char	*get_spell_vfx_str( spell_kind_t spell_kind )
 	case SPELL_KIND_SUMMON_WILL_O_WISP:
 	case SPELL_KIND_SUMMON_SHADE:
 		break;
+	case SPELL_KIND_DISMISSAL_ELEMENTAL:
+		return "><";
 	case SPELL_KIND_SHA_MAX_N:
 		break;
 /* BAR */
@@ -3626,5 +3682,28 @@ gold_t	get_price_spell( spell_kind_t kind )
 	return price;
 }
 
-/**/
+/***************************************************************
+* ꡼å
+***************************************************************/
 
+void	check_memory_spell_c( void )
+{
+	check_memory( check_memory_spell_c_cast_tab,
+			"spell.c: cast_tab" );
+	check_memory( check_memory_spell_c_g_sel_mbr,
+			"spell.c: g_sel_mbr" );
+	check_memory( check_memory_spell_c_g_dflt_spell_abl,
+			"spell.c: g_dflt_spell_abl" );
+	check_memory( check_memory_spell_c_g_dflt_spell,
+			"spell.c: g_dflt_spell" );
+	check_memory( check_memory_spell_c_g_dflt_spell_extent,
+			"spell.c: g_dflt_spell_extent" );
+	check_memory( check_memory_spell_c_g_sel_spell_abl,
+			"spell.c: g_sel_spell_abl" );
+	check_memory( check_memory_spell_c_g_vfx_n,
+			"spell.c: g_vfx_n" );
+	check_memory( check_memory_spell_c_g_vfx_bgn,
+			"spell.c: g_vfx_bgn" );
+}
+
+/**/
