@@ -17,6 +17,7 @@ bool_t	chk_shop_order( shop_n_t shop_n );
 char	*menu_order( void );
 char	*menu_order_sel_mbr( void );
 char	*cb_menu_order_sel( menu_t **pp, menu_t **dflt );
+void	next_g_sel_mbr( bool_t flg_loop );
 char	*cb_menu_order_chose( menu_t **pp, menu_t **dflt );
 bool_t	chk_order_milk( mbr_t *mbr, shop_n_t shop_n, item_t *item );
 char	*menu_shop( shop_n_t shop_n, mbr_t *mbr, char *str_goto );
@@ -75,8 +76,8 @@ char	*cb_menu_bar_arrange_up( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_down( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_top( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_bottom( menu_t **pp, menu_t **dflt );
-char	*cb_menu_bar_arrange_1( long d );
-char	*cb_menu_bar_arrange( long d );
+void	menu_bar_arrange_1( mbr_t *mbr, long d );
+void	menu_bar_arrange( mbr_t *mbr, long d );
 char	*cb_menu_bar_new_mbr( menu_t **pp, menu_t **dflt );
 bool_t	exec_menu_new_mbr( void );
 char	*cb_menu_bar_remove_mbr( menu_t **pp, menu_t **dflt );
@@ -88,19 +89,31 @@ char	*cb_menu_bar_arrange_up_class( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_down_class( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_top_class( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_arrange_bottom_class( menu_t **pp, menu_t **dflt );
+char	*cb_menu_bar_edit_skill( menu_t **pp, menu_t **dflt );
+char	*menu_edit_skill( long n );
 char	*cb_menu_bar_new_class( menu_t **pp, menu_t **dflt );
 char	*menu_new_class( long n );
 long	exec_edit_class( class_t *org, bool_t flg_new );
+long	exec_edit_skill( class_t *org, bool_t flg_new );
+void	set_edit_skill_n( edit_skill_t n );
+edit_skill_t	get_edit_skill_n( void );
+char	*exec_sel_skill_edit( class_t *work, long slot );
+char	*exec_sel_skill( skill_kind_t *ret_kind );
+char	*cb_menu_sel_skill( menu_t **pp, menu_t **dflt );
 char	*cb_menu_bar_remove_class( menu_t **pp, menu_t **dflt );
 rate_t	calc_abl_total( class_t *p );
 bool_t	exec_menu_ask_edit_class(
 	class_t *org, class_t *work, bool_t flg_new
 );
+bool_t	exec_menu_ask_edit_skill(
+	class_t *org, class_t *work, bool_t flg_new
+);
 void	menu_edit_class_inc_abl(
 	class_t *p, edit_class_t n, long add
 );
 char	*cb_menu_load_bar( menu_t **pp, menu_t **dflt );
 char	*cb_menu_save_bar( menu_t **pp, menu_t **dflt );
+void	menu_load_start( void );
 void	menu_load_bar( void );
 void	menu_save_bar( void );
 char	*menu_inn( void );
@@ -132,6 +145,7 @@ char	*cb_menu_auto_sale_chk( menu_t **pp, menu_t **dflt );
 char	*cb_menu_auto_sale_exec( menu_t **pp, menu_t **dflt );
 shop_n_t	get_cur_shop_n( void );
 void	set_cur_shop_n( shop_n_t n );
+void	check_memory_tmenu_c( void );
 
 #ifdef __cplusplus
 }
