@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2009 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: turn.c,v 1.41 2008/04/25 23:46:01 bowkenken Exp $
+* $Id: turn.c,v 1.45 2009/06/26 16:58:16 bowkenken Exp $
 ***************************************************************/
 
 /***************************************************************
@@ -40,6 +40,7 @@
 ***************************************************************/
 
 static long	g_day, g_turn;
+check_memory_def( check_memory_turn_c_g_turn )
 
 /***************************************************************
 * ν
@@ -218,6 +219,23 @@ long	get_game_day( void )
 	return g_day;
 }
 
+/***************************************************************
+* ߤΥ뤫Ĵ٤
+* return : νФ?
+***************************************************************/
+
+bool_t	chk_day( void )
+{
+	if( chk_morning() )
+		return FALSE;
+	if( chk_evening() )
+		return FALSE;
+	if( chk_night() )
+		return FALSE;
+
+	return TRUE;
+}
+
 /***************************************************************
 * ߤΥνФĴ٤
 * return : νФ?
@@ -344,7 +362,8 @@ char	*get_calendar_str( void )
 	minute = (turn / TURN_PER_MINUTE) % 60;
 	week = get_day_of_week_name();
 
-	sprintf( s, MSG_CALENDAR, year, month, day, week, hour, minute );
+	sn_printf( s, 127, MSG_CALENDAR,
+			year, month, day, week, hour, minute );
 
 	return s;
 }
@@ -364,7 +383,7 @@ char	*get_stat_time_str( void )
 	hour = (turn / TURN_PER_HOUR) % 24;
 	minute = (turn / TURN_PER_MINUTE) % 60;
 
-	sprintf( s, MSG_STAT_TIME, hour, minute );
+	sn_printf( s, 40, MSG_STAT_TIME, hour, minute );
 
 	return s;
 }
@@ -408,3 +427,12 @@ day_of_week_t	get_day_of_week( void )
 	return( (day_of_week_t)(get_game_day() % DAY_OF_WEEK_MAX_N) );
 }
 
+/***************************************************************
+* ꡼å
+***************************************************************/
+
+void	check_memory_turn_c( void )
+{
+	check_memory( check_memory_turn_c_g_turn,
+			"turn.c: g_turn" );
+}
